SystemOrganization addCategory: #'XML-Parser'!
SystemOrganization addCategory: #'XML-Parser-Exception'!
SystemOrganization addCategory: #'XML-Parser-Test'!
SystemOrganization addCategory: #'XML-Parser-DTD'!
SystemOrganization addCategory: #'XML-Parser-Nodes'!
SystemOrganization addCategory: #'XML-Parser-Parser'!

Warning subclass: #SAXWarning
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exception'!

TestCase subclass: #XMLNodeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Test'!

!XMLNodeTest methodsFor: 'assertions' stamp: 'JAAyer 2/25/2010 14:39'!
assertDocument: aDocument dtd: aDtd version: aVersion encoding: anEncoding required: aRequired
	self
		assert: aDocument dtd = aDtd;
		assert: aDocument version = aVersion;
		assert: aDocument encoding = anEncoding;
		assert: aDocument requiredMarkup = aRequired! !

!XMLNodeTest methodsFor: 'assertions' stamp: 'JAAyer 3/1/2010 01:31'!
assertElement: anElement attribute: aSymbol is: aValue
	self
		assert: (anElement includesAttribute: aSymbol);
		assert: (anElement attributeAt: aSymbol) = aValue;
		assert: (anElement at: aSymbol) = aValue;
		assert: (anElement @ aSymbol) = aValue! !

!XMLNodeTest methodsFor: 'assertions' stamp: 'Alexandre Bergel 3/4/2010 14:44'!
assertElement: anElement attributes: aDictionary
	self assert:
		anElement attributes =
			(anElement attributes species newFrom:
				(aDictionary associations collect: [:a | a key asSymbol -> a value]))! !

!XMLNodeTest methodsFor: 'assertions' stamp: 'JAAyer 2/26/2010 22:27'!
assertElement: anElement contentString: aContentString contents: aContentArray
	| contentStrings |

	self
		assert: anElement contentString = aContentString;
		assert: anElement characterData = aContentString.
	
	contentStrings := (anElement contents collect: [:each | each string]).
	self assert: contentStrings asArray = aContentArray.! !

!XMLNodeTest methodsFor: 'assertions' stamp: 'JAAyer 3/1/2010 16:50'!
assertElement: anElement name: aName attributes: aDictionary contentString: aString
	self
		assert: (anElement isNamed: aName);
		assertElement: anElement attributes: aDictionary;
		assert: anElement contentString = aString! !

!XMLNodeTest methodsFor: 'assertions' stamp: 'JAAyer 2/26/2010 22:38'!
assertElement: anElement name: aName localName: aLocalName
	self
		assert: anElement name == aName asSymbol;
		assert: anElement tag == aName asSymbol;
		assert: anElement qualifiedName == aName asSymbol;
		assert: anElement localName == aLocalName asSymbol;
		assert: (anElement isNamed: aName);
		assert: (anElement isNamed: aLocalName)! !

!XMLNodeTest methodsFor: 'assertions' stamp: 'JAAyer 2/25/2010 14:39'!
assertNode: aNode namespace: aNamespace uri: aUri
	self
		assert: aNode namespace = aNamespace;
		assert: aNode namespaceURI = aUri! !

!XMLNodeTest methodsFor: 'assertions' stamp: 'JAAyer 2/25/2010 14:39'!
assertPI: aNode target: aTarget data: aData
	self
		assert: aNode target = aTarget;
		assert: aNode data = aData;
		assert: aNode asString = ('<?', aTarget, ' ', aData, ' ?>')! !

!XMLNodeTest methodsFor: 'assertions' stamp: 'JAAyer 2/25/2010 14:39'!
assertString: aNode is: aString printed: aPrintString
	self
		assert: aNode string = aString;
		assert: aNode characterData = aString;
		assert: aNode asString = aPrintString! !

!XMLNodeTest methodsFor: 'accessing' stamp: 'JAAyer 2/28/2010 23:38'!
createNodes: aNumber with: aBlock
	^ ((1 to: aNumber) collect: [:i | aBlock valueWithPossibleArgument: i]) asOrderedCollection! !

!XMLNodeTest methodsFor: 'accessing' stamp: 'JAAyer 3/1/2010 01:11'!
escapedString
	^ 'test&lt;&gt;&amp;&quot;'! !

!XMLNodeTest methodsFor: 'accessing' stamp: 'JAAyer 3/1/2010 19:05'!
intersperse: aCollection with: aPaddingCollection
	| mixedCollection nodeStream paddingStream |

	mixedCollection := aCollection species new.
	nodeStream := aCollection readStream.
	paddingStream := aPaddingCollection readStream.
	[nodeStream atEnd and: [paddingStream atEnd]]
		whileFalse: [
			nodeStream atEnd
				ifFalse: [mixedCollection add: nodeStream next].
			paddingStream atEnd
				ifFalse: [mixedCollection add: paddingStream next]].
	^ mixedCollection.! !

!XMLNodeTest methodsFor: 'accessing' stamp: 'JAAyer 2/28/2010 23:52'!
interspersedWithNodes: aNodeCollection
	^ self
		intersperse: aNodeCollection
		with: (self newNodes: aNodeCollection size)! !

!XMLNodeTest methodsFor: 'accessing' stamp: 'JAAyer 3/1/2010 01:10'!
markupString
	^ 'test<>&"'! !

!XMLNodeTest methodsFor: 'accessing' stamp: 'JAAyer 2/28/2010 23:40'!
newElements: aNumber
	^ self
		createNodes: aNumber
		with: [:i | XMLElement named: 'test', i asString]! !

!XMLNodeTest methodsFor: 'accessing' stamp: 'JAAyer 3/3/2010 10:55'!
newElementsNamed: aNameArray
	| names fullName localName namespace |

	names := aNameArray asOrderedCollection.
	^ self
		createNodes: aNameArray size
		with: [
			fullName := names removeFirst asString.
			(localName := fullName copyAfter: $:)
				ifNotEmpty: [
					namespace := (fullName copyUpTo: $:).
					XMLElement
						named: localName
						namespace: namespace
						uri: 'http://', namespace
						attributes: Dictionary new]
				ifEmpty: [XMLElement named: fullName]].! !

!XMLNodeTest methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:39'!
newNodes: aNumber
	^ self createNodes: aNumber with: [XMLNode new]! !

!XMLNodeTest methodsFor: 'accessing' stamp: 'JAAyer 2/28/2010 23:40'!
newStrings: aNumber
	^ self
		createNodes: aNumber
		with: [:i | XMLStringNode string: 'test', i asString]! !

!XMLNodeTest methodsFor: 'assertions' stamp: 'JAAyer 2/25/2010 14:39'!
should: aBlock addNode: aNode to: aParent
	self should: aBlock addNodes: (Array with: aNode) to: aParent! !

!XMLNodeTest methodsFor: 'assertions' stamp: 'JAAyer 2/25/2010 14:39'!
should: aBlock addNodes: aNodeCollection to: aParent
	| added |

	self deny: (aNodeCollection anySatisfy: [:each | aParent includesNode: each]).
	(added := aBlock value) isCollection
		ifFalse: [added := Array with: added].
	self
		assert: added asArray = aNodeCollection asArray;
		assert: (added allSatisfy: [:each |
			(aParent includesNode: each) and: [each parent = aParent]])! !

!XMLNodeTest methodsFor: 'assertions' stamp: 'JAAyer 2/25/2010 14:39'!
should: aBlock enumerate: aCollection
	| enumerated |

	enumerated := OrderedCollection new.
	aBlock value: [:each | enumerated add: each].
	self assert: enumerated = aCollection asOrderedCollection.! !

!XMLNodeTest methodsFor: 'assertions' stamp: 'JAAyer 2/25/2010 14:39'!
should: aBlock removeNode: aNode from: aParent
	self should: aBlock removeNodes: (Array with: aNode) from: aParent! !

!XMLNodeTest methodsFor: 'assertions' stamp: 'JAAyer 2/25/2010 14:39'!
should: aBlock removeNodes: aNodeCollection from: aParent
	| removed |

	self assert: (aNodeCollection allSatisfy: [:each | aParent includesNode: each]).
	(removed := aBlock value) isCollection
		ifFalse: [removed := Array with: removed].

	self
		assert: removed asArray = aNodeCollection asArray;
		deny: (removed anySatisfy: [:each | aParent includesNode: each])! !

!XMLNodeTest methodsFor: 'tests' stamp: 'JAAyer 3/1/2010 20:54'!
test010node
	| node parent testingMessages accessors |

	node := XMLNode new.
	testingMessages := (XMLNode methodsInCategory: #testing) copyWithout: #isEmpty.
	accessors := #(nextNode previousNode parent).
	self
		assert: node isEmpty;
		deny: (testingMessages anySatisfy: [:each | node perform: each]);
		deny: (accessors anySatisfy: [:each | (node perform: each) notNil]).

	parent := XMLNode new.
	node parent: parent.
	self
		assert: node hasParent;
		assert: node parent = parent.

	self should: [node addContent: ''] raise: SAXParseException.! !

!XMLNodeTest methodsFor: 'tests' stamp: 'JAAyer 2/27/2010 21:23'!
test020processingInstruction
	| pi t d |

	pi := XMLPI new.
	t := 'target'.
	d := 'data'.
	self
		assert: pi isPI & pi isProcessingInstruction;
		assertPI: pi target: '' data: '';
		assertPI: (pi target: t) target: t data: '';
		assertPI: (pi data: d) target: t data: d;
		assertPI: (XMLPI target: t data: d) target: t data: d.! !

!XMLNodeTest methodsFor: 'tests' stamp: 'JAAyer 3/1/2010 17:35'!
test030stringNode
	| node raw escaped |

	node := XMLStringNode new.
	raw := self markupString.
	escaped := self escapedString.
	self
		assert: node isText & node isTagOrText;
		assertString: node is: '' printed: '';
		assertString: (node string: raw) is: raw printed: escaped;
		assertString: (node addContent: raw) is: raw, raw printed: escaped, escaped.
	self
		assertString: (node addContent: (XMLStringNode string: raw))
		is: raw, raw, raw
		printed: escaped, escaped, escaped.

	self assertString: (XMLStringNode string: raw) is: raw printed: escaped.
	self
		assertString: (XMLStringNode string: (XMLStringNode string: raw))
		is: raw
		printed: escaped.! !

!XMLNodeTest methodsFor: 'tests' stamp: 'JAAyer 2/25/2010 14:39'!
test040emptyNodeWithElements
	| node |

	node := XMLNodeWithElements new.
	self
		assert: node isEmpty;
		deny: node hasElements;
		assert: node nodes isEmpty;
		assert: node elements isEmpty;
		assert: node topNode isNil;
		assert: node topElement isNil.! !

!XMLNodeTest methodsFor: 'tests' stamp: 'JAAyer 3/3/2010 10:29'!
test041namespaces
	| node ns uri |

	node := XMLElement new.
	ns := 'foo'.
	uri := 'http://foo'.
	self
		assertNode: node namespace: nil uri: nil;
		assertNode: (node namespace: ns uri: uri) namespace: ns uri: uri.! !

!XMLNodeTest methodsFor: 'tests' stamp: 'JAAyer 3/1/2010 17:17'!
test042addingNodes
	| node children |

	node := XMLNodeWithElements new.
	children := self newNodes: 6.
	self
		should: [node addNode: children second] addNode: children second to: node;
		should: [node addNodeFirst: children first] addNode: children first to: node.
	self
		should: [node addNodes: (children copyFrom: 3 to: 4)]
		addNodes: (children copyFrom: 3 to: 4)
		to: node.
	self
		should: [node addNode: children sixth after: children fourth]
		addNode: children sixth
		to: node.
	self
		should: [node addNode: children fifth before: children sixth]
		addNode: children fifth
		to: node.

	self
		should: [node addNode: (XMLNode new parent: XMLNode new)] raise: Error;
		should: [node addNode: children first] raise: Error;
		assert: node nodes = children;
		assert: node topNode = children first.! !

!XMLNodeTest methodsFor: 'tests' stamp: 'JAAyer 2/25/2010 14:39'!
test043removingNodes
	| node children |

	node := XMLNodeWithElements new.
	children := self newNodes: 5.
	node addNodes: children.
	self
		should: [node removeNode: children first]
		removeNode: children first
		from: node.
	self
		should: [node removeNodes: (children copyFrom: 2 to: 3)]
		removeNodes: (children copyFrom: 2 to: 3)
		from: node.
	self
		should: [node removeNodes]
		removeNodes: (children last: 2)
		from: node.
	self assert: node isEmpty.! !

!XMLNodeTest methodsFor: 'tests' stamp: 'JAAyer 2/25/2010 14:39'!
test044accessingBeforeAfterNodes
	| node children |

	node := XMLNodeWithElements new.
	children := self newNodes: 3.
	node addNodes: children.
	children do: [:each | | next previous |
		next := children after: each ifAbsent: [nil].
		previous := children before: each ifAbsent: [nil].
		
		self
			assert: each nextNode = next;
			assert: each previousNode = previous;
			assert: (node nodeAfter: each) = next;
			assert: (node nodeBefore: each) = previous].! !

!XMLNodeTest methodsFor: 'tests' stamp: 'JAAyer 2/27/2010 15:17'!
test050emptyElement
	| element |

	element := XMLElement new.
	self
		assert: element isTag & element isTagOrText & element isElement;
		deny: element hasAttributes;
		deny: element hasContents;
		assert: element elementsAndContents isEmpty;
		assert: element contents isEmpty.! !

!XMLNodeTest methodsFor: 'tests' stamp: 'JAAyer 2/26/2010 22:41'!
test051elementNaming
	| element |

	element := XMLElement new.
	self
		assertElement: element name: '' localName: '';
		assertElement: (element name: 'foo') name: #foo localName: #foo;
		assertNode: element namespace: nil uri: nil.

	element namespace: 'bar' uri: 'http://bar'.
	self
		assertElement: element name: #bar:foo localName: #foo;
		assertNode: element namespace: 'bar' uri: 'http://bar'.

	element name: 'baz' namespace: 'foo' uri: 'http://foo'.
	self
		assertElement: element name: #foo:baz localName: #baz;
		assertNode: element namespace: 'foo' uri: 'http://foo'.! !

!XMLNodeTest methodsFor: 'tests' stamp: 'JAAyer 3/1/2010 17:53'!
test052elementContentString
	| element str |

	str := self markupString.
	(element := XMLElement new) contentString: str.
	self
		deny: element isEmpty;
		assert: element hasContents;
		assert: element contentString = str.

	"Intersperse between text"
	element addNode: XMLNode new.
	self
		assertElement: (element addContent: str)
		contentString: str
		contents: (Array with: str with: str).

	"addContent: should combine neighboring string nodes"
	self
		assertElement: (element addContent: (XMLStringNode string: str))
		contentString: str
		contents: (Array with: str with: str, str).

	element addElement: (XMLElement named: 'foo' contentString: 'test').
	self assert: (element contentStringAt: #foo) = 'test'.! !

!XMLNodeTest methodsFor: 'tests' stamp: 'Alexandre Bergel 3/4/2010 08:53'!
test053elementAttributes
	| element attrs |

	element := XMLElement new.
	self
		assert: (element at: #foo) isEmpty;
		assert: (element attributeAt: #foo) isNil;
		assert: (element at: #foo ifAbsent: ['absent']) = 'absent';
		assert: (element attributeAt: #foo ifAbsent: ['absent']) = 'absent';
		deny: (element includesAttribute: #foo).

	self
		assert: (element attributeAt: #foo put: 'test') = 'test';
		assert: element hasAttributes;
		assertElement: element attribute: 'foo' is: 'test';
		assert: (element attributeAt: 'bar' put: 'test2') = 'test2';
		assertElement: element attribute: #bar is: 'test2';
		assert: (element at: 'baz' put: 'test3') = 'test3';
		assertElement: element attribute: #baz is: 'test3';
		assert: element allAttributes asArray =  #(#bar #baz #foo).

	self
		assert: element allAttributeAssociations asArray =
			(Array with: #bar->'test2' with: #baz->'test3' with: #foo->'test').
	self
		assertElement: element
		attributes: (Dictionary with: #foo->'test' with: #bar->'test2' with: #baz->'test3').

	self
		assert: (element removeAttribute: #foo) = 'test';
		assert: (element removeAttribute: 'bar') = 'test2';
		assert: (element removeAttribute: #baz) = 'test3';
		deny: element hasAttributes.! !

!XMLNodeTest methodsFor: 'tests' stamp: 'JAAyer 3/1/2010 04:31'!
test054elementConstructors
	| noAttributes attributes str element |

	noAttributes := Dictionary new.
	self
		assertElement: (XMLElement named: #foo)
		name: #foo
		attributes: noAttributes
		contentString: ''.

	attributes := Dictionary with: #foo->'test1' with: 'bar'->'test2'.
	self
		assertElement: (XMLElement named: #foo attributes: attributes)
		name: #foo
		attributes: attributes
		contentString: ''.

	str := self markupString.
	self
		assertElement: (XMLElement named: #foo contentString: str)
		name: #foo
		attributes: noAttributes
		contentString: str.
	self
		assertElement: (XMLElement named: #foo attributes: attributes contentString: str)
		name: #foo
		attributes: attributes
		contentString: str.

	element := XMLElement named: #foo namespace: 'prefix' uri: 'http://foo' attributes: attributes.
	self
		assertElement: element name: #prefix:foo attributes: attributes contentString: '';
		assertNode: element namespace: 'prefix' uri: 'http://foo'.! !

!XMLNodeTest methodsFor: 'tests' stamp: 'JAAyer 2/25/2010 14:39'!
test060addingElements
	| node elements children |

	node := XMLNodeWithElements new.
	elements := self newElementsNamed: #(foo bar baz).
	node addNodes: (self interspersedWithNodes: elements).
	self
		assert: node hasElements;
		assert: node topElement = elements first;
		assert: node elements = elements;
		assert: (elements allSatisfy: [:each | node includesElement: each name]).! !

!XMLNodeTest methodsFor: 'tests' stamp: 'JAAyer 2/25/2010 14:39'!
test061accessingBeforeAfterElements
	| node elements |

	node := XMLNodeWithElements new.
	elements := self newElements: 3.
	node addNodes: (self interspersedWithNodes: elements).
	elements do: [:each | | next previous |
		next := elements after: each ifAbsent: [nil].
		previous := elements before: each ifAbsent: [nil].
		
		self
			assert: each nextElement = next;
			assert: each previousElement = previous;
			assert: (node elementAfter: each) = next;
			assert: (node elementBefore: each) = previous].! !

!XMLNodeTest methodsFor: 'tests' stamp: 'Alexandre Bergel 3/4/2010 08:53'!
test062elementPrinting
	| element attributes |

	element := (XMLElement named: #foo).
	attributes := Dictionary with: #a->'one' with: #b->'two'.
	self
		assert: element asString = '<foo></foo>';
		assert: (element contentString: 'test') printString = '<foo>test</foo>';
		assert: (element setAttributes: attributes) printString = '<foo a="one" b="two">test</foo>';
		assert: (element equalTo: (XMLElement
			named: 'foo'
			attributes: (Dictionary with: #a->'one' with: #b->'two')
			contentString: 'test'))! !

!XMLNodeTest methodsFor: 'tests' stamp: 'JAAyer 2/26/2010 21:01'!
test070nodeAndElementEnumeration
	| node nodes elements |

	node := XMLNodeWithElements new.
	self
		should: [:block | node nodesDo: block] enumerate: #();
		should: [:block | node elementsDo: block] enumerate: #().

	elements := self newElements: 3.
	nodes := self interspersedWithNodes: elements.
	node addNodes: nodes.
	self
		should: [:block | node nodesDo: block] enumerate: nodes;
		should: [:block | node elementsDo: block] enumerate: elements.! !

!XMLNodeTest methodsFor: 'tests' stamp: 'JAAyer 2/28/2010 23:54'!
test071elementsAndContentsEnumeration
	| node nodes elements contents elementsAndContents |

	node := XMLElement new.
	elements := self newElements: 3.
	contents := self newStrings: 3.
	elementsAndContents := self intersperse: elements with: contents.
	node addNodes: (self interspersedWithNodes: elementsAndContents).

	self
		assert: node elementsAndContents = elementsAndContents;
		should: [:block | node elementsAndContentsDo: block] enumerate: elementsAndContents;
		should: [:block | node contentsDo: block] enumerate: contents.! !

!XMLNodeTest methodsFor: 'tests' stamp: 'JAAyer 3/1/2010 01:45'!
test080elementAccessing
	| node nodes elements |

	node := XMLNodeWithElements new.
	elements := self newElementsNamed: #(foo bar prefix:foo baz).
	node addNodes: (self interspersedWithNodes: elements).
	self
		assert: (node elementAt: #absent) isNil;
		assert: (node elementAt: #absent ifAbsent: ['absent']) = 'absent';
		assert: (node elementsAt: #absent) isEmpty;
		assert: (node elementsAt: #absent ifAbsent: ['absent']) = 'absent';
		assert: (node elementAtAny: #(absent)) isNil;
		assert: (node elementAtAny: #(absent) ifAbsent: ['absent']) = 'absent';
		assert: (node elementAt: 'foo') = elements first;
		assert: (node elementAt: #bar) = elements second;
		assert: (node elementsAt: #foo) = ((elements first: 3) copyWithout: elements second);
		assert: (node elementAt: 'prefix:foo') = elements third;
		assert: (node elementsAt: 'prefix:foo') = (OrderedCollection with: elements third);
		assert: (node elementAtAny: #(bar prefix:foo)) = elements second;
		assert: (node elementAtAny: #(absent 'baz')) = elements last.! !

!XMLNodeTest methodsFor: 'tests' stamp: 'JAAyer 3/1/2010 17:48'!
test081firstTagSearching
	| node elements |

	node := XMLNodeWithElements new.
	elements := self newElementsNamed: #(foo bar prefix:foo prefix:bar bar baz).
	node addNodes: (elements first: 2).
	node nodes first addNodes: (elements copyFrom: 3 to: 4).
	node nodes second addNodes: (elements copyFrom: 5 to: 6).
	self
		assert: (node firstTagNamed: #absent) isNil;
		assert: (node firstTagNamed: #foo with: [false]) isNil;
		assert: (node firstTagNamedAny: #(absent)) isNil;
		assert: (node firstTagNamedAny: #(foo) with: [false]) isNil.

	self
		assert: (node firstTagNamed: 'foo') = elements first;
		assert: (node firstTagNamed: #prefix:bar) = elements fourth;
		assert: (node firstTagNamedAny: #(prefix:foo prefix:bar absent)) = elements third;
		assert: (node firstTagNamedAny: #(absent:bar 'baz')) = elements last;
		assert: (elements first firstTagNamed: #foo) = elements first;
		assert: (elements second firstTagNamedAny: #(foo bar)) = elements second.
	self assert: (node
		firstTagNamed: #bar
		with: [:each | each = elements fifth]) = elements fifth.
	self assert: (node
		firstTagNamedAny: #(bar 'baz')
		with: [:each | each = elements last]) = elements last.
	self assert: (elements first
		firstTagNamed: #foo
		with: [:each | each = elements third]) = elements third.
	self assert: (elements second
		firstTagNamedAny: #(bar)
		with: [:each | each = elements fifth]) = elements fifth.! !

!XMLNodeTest methodsFor: 'tests' stamp: 'JAAyer 3/1/2010 17:48'!
test082tagsNamedSearching
	| node elements |

	node := XMLNodeWithElements new.
	elements := self newElementsNamed: #(foo bar prefix:foo bar prefix:bar baz).
	node addNodes: (elements first: 2).
	node nodes first addNodes: (elements copyFrom: 3 to: 4).
	node nodes second addNodes: (elements copyFrom: 5 to: 6).
	self
		should: [:block | node tagsNamed: #absent do: block] enumerate: #();
		should: [:block | node elementsAt: #absent do: block] enumerate: #();
		should: [:block | node tagsNamed: #absent childrenDoAndRecurse: block] enumerate: #();
		should: [:block | node tagsNamed: #absent contentsDo: block] enumerate: #();
		should: [:block | node tagsNamed: #foo contentsDo: block] enumerate: #().

	self
		should: [:block | node tagsNamed: #foo do: block]
		enumerate: (Array with: elements first with: elements third).
	self
		should: [:block | node tagsNamed: 'prefix:foo' do: block]
		enumerate: (Array with: elements third).
	self
		should: [:block | node topElement elementsAt: #foo do: block]
		enumerate: (Array with: elements third).
	self
		should: [:block | node tagsNamed: #bar childrenDoAndRecurse: block]
		enumerate: (Array with: elements fourth with: elements second with: elements fifth).

	elements first
		addContent: 'testing 1';
		addNode: XMLNode new; "Intersperse between string nodes"
		addContent: 'testing 2'.
	elements third addContent: 'testing'.
	self
		should: [:block | node tagsNamed: #foo contentsDo: block]
		enumerate: elements first contents, elements third contents.! !

!XMLNodeTest methodsFor: 'tests' stamp: 'JAAyer 3/1/2010 01:53'!
test083elementRemoving
	| node nodes elements |

	node := XMLNodeWithElements new.
	elements := self newElementsNamed: #(foo prefix:bar bar baz).
	node addNodes: (self interspersedWithNodes: elements).
	self
		assert: (node removeNode: elements first) = elements first;
		assert: node elements = elements allButFirst;
		assert: (node elementAt: #foo) isNil;
		assert: (node elementsAt: #foo) isEmpty;
		assert: (node removeNode: elements second) = elements second;
		assert: node elements = (elements last: 2);
		assert: (node elementAt: #bar) = elements third;
		assert: (node elementsAt: #bar) = (OrderedCollection with: elements third);
		assert: (node removeNode: elements third) = elements third;
		assert: node elements = (elements last: 1);
		assert: (node elementAt: #bar) isNil;
		assert: (node elementsAt: #bar) isEmpty;
		assert: node topElement = elements last;
		assert: (node removeNode: elements last) = elements last;
		deny: node hasElements;
		assert: node topElement isNil.! !

!XMLNodeTest methodsFor: 'tests' stamp: 'JAAyer 2/26/2010 20:50'!
test084elementRenaming
	| node nodes elements |

	node := XMLNodeWithElements new.
	elements := self newElementsNamed: #(foo prefix:bar bar baz).
	node addNodes: (self interspersedWithNodes: elements).

	elements first name: #notfoo.
	self
		deny: (node includesElement: #foo);
		assert: (node elementAt: #foo) isNil;
		assert: (node elementsAt: #foo) isEmpty;
		assert: (node elementAt: #notfoo) = elements first.

	elements third namespace: 'prefix' uri: ''.
	self assert: (node elementsAt: #prefix:bar) = (elements copyFrom: 2 to: 3).

	elements fourth name: #bar.
	self assert: (node elementsAt: #bar) = elements allButFirst.

	elements fourth name: #foo.
	self assert: (node elementAt: #foo) = elements last.! !

!XMLNodeTest methodsFor: 'tests' stamp: 'JAAyer 3/1/2010 17:13'!
test090documentTest
	| doc root |

	doc := XMLDocument new.
	root := XMLElement new.
	self
		assert: doc isDocument;
		assert: doc root isNil;
		assert: (doc addElement: root) root = root;
		assertDocument: doc dtd: nil version: nil encoding: 'UTF-8' required: nil.

	self
		assertDocument: (doc dtd: '<!!DOCTYPE root>')
		dtd: '<!!DOCTYPE root>'
		version: nil
		encoding: 'UTF-8'
		required: nil.
	self
		assertDocument: (doc version: '1.0')
		dtd: '<!!DOCTYPE root>'
		version: '1.0'
		encoding: 'UTF-8'
		required: nil.
	self
		assertDocument: (doc encoding: 'ISO Latin-1')
		dtd: '<!!DOCTYPE root>'
		version: '1.0'
		encoding: 'ISO Latin-1'
		required: nil.
	self
		assertDocument: (doc requiredMarkup: 'root')
		dtd: '<!!DOCTYPE root>'
		version: '1.0'
		encoding: 'ISO Latin-1'
		required: 'root'.! !

TestCase subclass: #XMLParserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Test'!

!XMLParserTest methodsFor: 'source' stamp: 'AlexandreBergel 1/29/2009 11:49'!
addressBookXML
	^'<addressbook>
  <person employee-number="A0000" family-name="Gates" first-name="Bob">
    <contact-info><!!--Confidential--></contact-info>
    <address city="Los Angeles" number="1239" state="CA" street="Pine Rd."/>
    <job-info employee-type="Full-Time" is-manager="no" job-description="Manager"/>
    <manager employee-number="A0000"/>
  </person>
  <person employee-number="A7000" family-name="Brown"
    first-name="Robert" middle-initial="L.">
    <contact-info>
      <email address="robb@iro.ibm.com"/>
      <home-phone number="03-3987873"/>
    </contact-info>
    <address city="New York" number="344" state="NY" street="118 St."/>
    <job-info employee-type="Full-Time" is-manager="yes" job-description="Group Leader"/>
    <manager employee-number="A0000"/>
  </person>
  <person employee-number="A7890" family-name="DePaiva"
    first-name="Kassie" middle-initial="W.">
    <contact-info><!!-- Kassie''s agent phone: 03-987654 --></contact-info>
    <address city="Los Angeles" number="1234" state="CA" street="Pine Rd."/>
    <job-info employee-type="Full-Time" is-manager="no" job-description="Actor"/>
    <manager employee-number="A0000"/>
    <misc-info>One of the most talented actresses on Daytime. Kassie
      plays the devious and beautiful Blair Cramer on ABC&apos;s
      &quot;One Life To Live.&quot;</misc-info>
  </person>
  <person employee-number="A7987" family-name="Smith" first-name="Joe">
    <contact-info>
      <email address="joes@iro.ibm.com"/>
      <mobile-phone number="888-7657765"/>
      <home-phone number="03-8767898"/>
      <home-phone number="03-8767871"/>
    </contact-info>
    <address city="New York" number="12789" state="NY" street="W. 15th Ave."/>
    <job-info employee-type="Part-Time" is-manager="no" job-description="Hacker"/>
    <manager employee-number="A7000"/>
  </person>
</addressbook>
'! !

!XMLParserTest methodsFor: 'source' stamp: 'AlexandreBergel 1/29/2009 11:49'!
addressBookXMLWithDTD
	^'<?xml version="1.0" encoding="UTF-8"?>
<!!DOCTYPE addressbook SYSTEM "addressbook.dtd">
<?xml-stylesheet type="text/xsl" href="demo.xsl"?>
<addressbook>
  <person employee-number="A0000" family-name="Gates" first-name="Bob">
    <contact-info><!!--Confidential--></contact-info>
    <address city="Los Angeles" number="1239" state="CA" street="Pine Rd."/>
    <job-info employee-type="Full-Time" is-manager="no" job-description="Manager"/>
    <manager employee-number="A0000"/>
  </person>
  <person employee-number="A7000" family-name="Brown"
    first-name="Robert" middle-initial="L.">
    <contact-info>
      <email address="robb@iro.ibm.com"/>
      <home-phone number="03-3987873"/>
    </contact-info>
    <address city="New York" number="344" state="NY" street="118 St."/>
    <job-info employee-type="Full-Time" is-manager="yes" job-description="Group Leader"/>
    <manager employee-number="A0000"/>
  </person>
  <person employee-number="A7890" family-name="DePaiva"
    first-name="Kassie" middle-initial="W.">
    <contact-info><!!-- Kassie''s agent phone: 03-987654 --></contact-info>
    <address city="Los Angeles" number="1234" state="CA" street="Pine Rd."/>
    <job-info employee-type="Full-Time" is-manager="no" job-description="Actor"/>
    <manager employee-number="A0000"/>
    <misc-info>One of the most talented actresses on Daytime. Kassie
      plays the devious and beautiful Blair Cramer on ABC&apos;s
      &quot;One Life To Live.&quot;</misc-info>
  </person>
  <person employee-number="A7987" family-name="Smith" first-name="Joe">
    <contact-info>
      <email address="joes@iro.ibm.com"/>
      <mobile-phone number="888-7657765"/>
      <home-phone number="03-8767898"/>
      <home-phone number="03-8767871"/>
    </contact-info>
    <address city="New York" number="12789" state="NY" street="W. 15th Ave."/>
    <job-info employee-type="Part-Time" is-manager="no" job-description="Hacker"/>
    <manager employee-number="A7000"/>
  </person>
</addressbook>
'! !

!XMLParserTest methodsFor: 'benchmark' stamp: 'Alexandre Bergel 3/4/2010 14:59'!
benchmark1
	"
	XMLParserTest new benchmark1     

	Results below are obtained on a MacBookPro, VM 4.2.1betaU1, 2Go RAM, 2.26 Ghz
	2097 => Version 45 (Alexandre Bergel 3/4/2010 14:54)
	"
	Smalltalk garbageCollect.
	^ [ 300 timesRepeat: [ self testParsing ]] timeToRun! !

!XMLParserTest methodsFor: 'code test' stamp: 'Alexandre Bergel 2/26/2010 13:45'!
categoriesForClass: aClass

 ^ aClass organization allMethodSelectors collect: 
			[:each |  aClass organization categoryOfElement: each].
! !

!XMLParserTest methodsFor: 'code test' stamp: 'Alexandre Bergel 2/26/2010 13:42'!
targetClass
	^ XMLParser! !

!XMLParserTest methodsFor: 'code test' stamp: 'Alexandre Bergel 2/26/2010 13:43'!
testClassComment
	"Check whether XMLParser has some comments"
	self should: [self targetClass organization hasComment].! !

!XMLParserTest methodsFor: 'tests' stamp: 'AlexandreBergel 1/29/2009 11:52'!
testExampleAddressBook
	| tokenizer |
	"self debug: #testExampleAddressBook"

	tokenizer := XMLTokenizer on: self addressBookXML readStream.

	"We enumerate the first characters of the addressbook example. The file being parsed begins with <addressbook"
	self assert: tokenizer next = $<.
	self assert: tokenizer next = $a.	
	self assert: tokenizer next = $d.
	self assert: tokenizer next = $d.
	self assert: tokenizer next = $r.

	self shouldnt: ([tokenizer next notNil] whileTrue: []) raise: Error. ! !

!XMLParserTest methodsFor: 'tests' stamp: 'AlexandreBergel 1/29/2009 11:53'!
testExampleAddressBookWithDTD
	| tokenizer |
	"XMLTokenizer exampleAddressBookWithDTD"

	tokenizer := XMLTokenizer on: self addressBookXMLWithDTD readStream.
	self shouldnt: ([tokenizer next notNil] whileTrue: []) raise: Error! !

!XMLParserTest methodsFor: 'tests' stamp: 'GeorgeHerolyants 1/8/2010 20:50'!
testLineEndingsDoNotMatter
   |text cr crlf|
   text :=
 '<foo>
 bar
 baz
 </foo>'.
	cr := XMLDOMParser parseDocumentFrom: text readStream.
	crlf := XMLDOMParser parseDocumentFrom: text withInternetLineEndings readStream.
	self assert:
		(cr elements first contentString) =
		(crlf elements first contentString).
! !

!XMLParserTest methodsFor: 'tests' stamp: 'JAAyer 3/2/2010 20:05'!
testParsing
	| xmlDocument root firstPerson numberOfPersons |
	"self debug: #testParsing"

	xmlDocument := XMLDOMParser parseDocumentFrom: self addressBookXML readStream.
	self assert: (xmlDocument isKindOf: XMLDocument).
	root := xmlDocument root.
	self assert: (root class == XMLElement).
	
	"the tag need not be a symbol!!"
	self assert: (root firstTagNamed: 'person') notNil.
	self assert: (root firstTagNamed: 'addressbook') notNil.

	self assert: (root firstTagNamed: #addressbook) == root.

	numberOfPersons := 0.
	root tagsNamed: #person do: [:p | numberOfPersons := numberOfPersons + 1].
	self assert: numberOfPersons = 4.

	firstPerson := root firstTagNamed: #person.
	self assert: (firstPerson attributeAt: #'employee-number') = 'A0000'.
	self assert: (firstPerson attributeAt: #'family-name') = 'Gates'.
	self assert: (firstPerson attributeAt: #'first-name') = 'Bob'.
	self assert: firstPerson allAttributes asArray =
		#(#'employee-number' #'family-name' #'first-name').
	self assert: firstPerson allAttributeAssociations asArray =
		{(#'employee-number'->'A0000'). (#'family-name'->'Gates'). (#'first-name'->'Bob').}.

	self assert: (firstPerson @ #'employee-number') = (firstPerson attributeAt: #'employee-number').
	self assert: (firstPerson @ #'employee-number') = (firstPerson @ 'employee-number').! !

!XMLParserTest methodsFor: 'tests' stamp: 'AlexandreBergel 1/29/2009 12:21'!
testParsingCharacters
	| parser |
	"This test is actually not that useful. This is not the proper way of using the parser. This test is here just for specification purpose"
	"self debug: #testParsingCharacters"

	parser := XMLParser on: self addressBookXML readStream.

	self assert: parser next = $<.
	self assert: parser next = $a.	
	self assert: parser next = $d.
	self assert: parser next = $d.
	self assert: parser next = $r.! !

!XMLParserTest methodsFor: 'xmltokenizer' stamp: 'Alexandre Bergel 2/24/2010 11:16'!
testTokenizerAddressBook
	| tokenizer |
	tokenizer := XMLTokenizer on: self addressBookXML readStream.
	[tokenizer next notNil]
		whileTrue: []! !

!XMLParserTest methodsFor: 'xmltokenizer' stamp: 'Alexandre Bergel 2/24/2010 11:16'!
testTokenizerAddressBookWithDTD
	| tokenizer |
	tokenizer := XMLTokenizer on: self addressBookXMLWithDTD readStream.
	[tokenizer next notNil]
		whileTrue: []! !

!XMLParserTest methodsFor: 'code test' stamp: 'Alexandre Bergel 2/26/2010 13:43'!
testUnCategorizedMethods
	| categories slips  |
	categories := self categoriesForClass: self targetClass.
	slips := categories select: [:each | each = #'as yet unclassified'].
	self should: [slips isEmpty].	! !

!ByteString methodsFor: '*xml-parser' stamp: 'mir 7/14/2006 11:54'!
applyLanguageInfomation: languageEnvironment
! !

Object subclass: #DTDEntityDeclaration
	instanceVariableNames: 'name value ndata'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

DTDEntityDeclaration class
	instanceVariableNames: 'contextBehavior'!

DTDEntityDeclaration class
	instanceVariableNames: 'contextBehavior'!

!DTDEntityDeclaration class methodsFor: 'accessing' stamp: 'mir 11/16/2000 20:14'!
behaviorForContext: aContext
	^self contextBehavior at: aContext! !

!DTDEntityDeclaration class methodsFor: 'accessing' stamp: 'mir 11/16/2000 20:15'!
contextBehavior
	^contextBehavior! !

!DTDEntityDeclaration class methodsFor: 'class initialization' stamp: 'mir 1/15/2002 18:02'!
initialize
	"DTDEntityDeclaration initialize"

	contextBehavior := Dictionary new.
	contextBehavior
		at: #content put: #include ;
		at: #attributeValueContent put: #includedInLiteral ;
		at: #attributeValue put: #forbidden ;
		at: #entityValue put: #bypass ;
		at: #dtd put: #forbidden ! !

!DTDEntityDeclaration class methodsFor: 'accessing' stamp: 'mir 11/16/2000 20:27'!
leadIn
	^'&'! !

!DTDEntityDeclaration class methodsFor: 'instance creation' stamp: 'mir 11/16/2000 20:13'!
name: aString value: aValueString
	^self new
		name: aString;
		value: aValueString! !

!DTDEntityDeclaration methodsFor: 'behaviors' stamp: 'mir 1/15/2002 11:29'!
bypass
	"Return my reference as is."
	^self reference! !

!DTDEntityDeclaration methodsFor: 'behaviors' stamp: 'mir 1/15/2002 11:29'!
forbidden
	self error: 'Forbidden reference usage'! !

!DTDEntityDeclaration methodsFor: 'behaviors' stamp: 'mir 1/15/2002 18:01'!
include
	"Return my expanded value."
	^value ifNil: [SAXWarning signal: 'XML undefined entity ' , name printString]! !

!DTDEntityDeclaration methodsFor: 'behaviors' stamp: 'mir 1/15/2002 18:06'!
includedInLiteral
	"Return my expanded value."
	^self include! !

!DTDEntityDeclaration methodsFor: 'accessing' stamp: 'mir 1/4/2002 19:40'!
name
	^name! !

!DTDEntityDeclaration methodsFor: 'accessing' stamp: 'mir 1/17/2002 15:25'!
name: aString
	name := aString asSymbol! !

!DTDEntityDeclaration methodsFor: 'accessing' stamp: 'mir 12/8/2000 17:22'!
ndata
	^ndata! !

!DTDEntityDeclaration methodsFor: 'accessing' stamp: 'mir 12/8/2000 17:22'!
ndata: aString
	ndata := aString! !

!DTDEntityDeclaration methodsFor: 'behaviors' stamp: 'mir 1/15/2002 11:30'!
reference
	"Return my reference as is."
	^self class leadIn , self name , ';'! !

!DTDEntityDeclaration methodsFor: 'invocation' stamp: 'mir 11/16/2000 21:23'!
registerIn: aParser
	aParser entity: self name put: self! !

!DTDEntityDeclaration methodsFor: 'accessing' stamp: 'mir 11/16/2000 10:54'!
value
	^value! !

!DTDEntityDeclaration methodsFor: 'accessing' stamp: 'mir 11/16/2000 10:55'!
value: aString
	value := aString! !

!DTDEntityDeclaration methodsFor: 'invocation' stamp: 'mir 1/15/2002 15:08'!
valueForContext: aContext
	^self perform: (self class behaviorForContext: aContext)! !

DTDEntityDeclaration subclass: #DTDExternalEntityDeclaration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDExternalEntityDeclaration classSide methodsFor: 'class initialization' stamp: 'mir 1/14/2002 18:15'!
initialize
	"DTDExternalEntityDeclaration initialize"

	contextBehavior := Dictionary new.
	contextBehavior
		at: #content put: #include ;
		at: #attributeValueContent put: #includedInLiteral ;
		at: #attributeValue put: #forbidden ;
		at: #entityValue put: #bypass ;
		at: #dtd put: #forbidden ! !

DTDEntityDeclaration subclass: #DTDParameterEntityDeclaration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDParameterEntityDeclaration class methodsFor: 'class initialization' stamp: 'mir 1/14/2002 18:15'!
initialize
	"DTDParameterEntityDeclaration initialize"

	contextBehavior := Dictionary new.
	contextBehavior
		at: #content put: #notRecognized: ;
		at: #attributeValueContent put: #notRecognized: ;
		at: #attributeValue put: #notRecognized: ;
		at: #entityValue put: #include: ;
		at: #dtd put: #includePE:! !

!DTDParameterEntityDeclaration class methodsFor: 'accessing' stamp: 'mir 11/16/2000 20:27'!
leadIn
	^'%'! !

!DTDParameterEntityDeclaration methodsFor: 'behaviors' stamp: 'mir 1/15/2002 11:30'!
includePE
	"Return my expanded value."
	^self include! !

!DTDParameterEntityDeclaration methodsFor: 'behaviors' stamp: 'mir 1/15/2002 23:21'!
notRecognized
	SAXMalformedException signal: 'Malformed entity.'! !

!DTDParameterEntityDeclaration methodsFor: 'invocation' stamp: 'mir 11/28/2000 17:26'!
registerIn: aParser
	aParser parameterEntity: self name put: self! !

Object subclass: #SAXHandler
	instanceVariableNames: 'document driver eod'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Parser'!

!SAXHandler commentStamp: 'JAAyer 2/21/2010 17:56' prior: 0!
A SAXHandler is an XML parser that parses XML using event handling. The acronym "SAX" refers to Java's "Simple API for XML," on which SAXHandler's API is based. To use this class, create a subclass and override handlers in "content" and "lexical" as needed.!

!SAXHandler class methodsFor: 'instance creation' stamp: 'sd 5/23/2003 15:19'!
on: aStream
	| driver parser |
	driver := SAXDriver on: aStream.
	driver validating: true.
	parser := self new driver: driver.
	^parser! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'cwp 5/28/2003 02:08'!
parseDTDFrom: aStream
	| driver parser |
	driver := SAXDriver on: aStream.
	driver validating: true.
	driver startParsingMarkup.
	parser := self new driver: driver.
	parser startDocument.
	parser parseDocument.
	^parser! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'JAAyer 3/3/2010 22:00'!
parseDocumentFrom: aStream
	^self parseDocumentFrom: aStream useNamespaces: true! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'mir 7/12/2006 16:26'!
parseDocumentFrom: aStream useNamespaces: aBoolean
	|  parser |
	parser := self on: aStream.
	parser useNamespaces: aBoolean.
	parser startDocument.
	parser parseDocument.
	^parser! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'mir 1/8/2002 15:55'!
parseDocumentFromFileNamed: fileName
	^self parseDocumentFromFileNamed: fileName readIntoMemory: false! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'mir 1/8/2002 15:55'!
parseDocumentFromFileNamed: fileName readIntoMemory: readIntoMemory
	| stream xmlDoc |
	stream := FileDirectory default readOnlyFileNamed: fileName.
	readIntoMemory
		ifTrue: [stream := stream contentsOfEntireFile readStream].
	[xmlDoc := self parseDocumentFrom: stream]
		ensure: [stream close].
	^xmlDoc! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'mir 6/5/2003 19:15'!
parserOnFileNamed: fileName
	^self parserOnFileNamed: fileName readIntoMemory: false! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'mir 6/5/2003 19:14'!
parserOnFileNamed: fileName readIntoMemory: readIntoMemory
	| stream  |
	stream := FileDirectory default readOnlyFileNamed: fileName.
	readIntoMemory
		ifTrue: [stream := stream contentsOfEntireFile readStream].
	^self on: stream! !

!SAXHandler methodsFor: 'content' stamp: 'mir 1/8/2002 18:27'!
characters: aString
	"This call corresponds to the Java SAX call
	characters(char[] ch, int start, int length)."! !

!SAXHandler methodsFor: 'content' stamp: 'mir 1/8/2002 18:24'!
checkEOD
	"Check if the document shouldn't be ended already"
	self eod
		ifTrue: [self driver errorExpected: 'No more data expected,']! !

!SAXHandler methodsFor: 'lexical' stamp: 'mir 8/11/2000 18:52'!
comment: commentString
	"This call corresponds to the Java SAX ext call
	comment(char[] ch, int start, int length)."! !

!SAXHandler methodsFor: 'accessing' stamp: 'mir 11/30/2000 18:12'!
document
	^document! !

!SAXHandler methodsFor: 'accessing' stamp: 'mir 11/30/2000 18:12'!
document: aDocument
	document := aDocument! !

!SAXHandler methodsFor: 'content' stamp: 'mir 1/17/2002 13:12'!
documentAttributes: attributeList! !

!SAXHandler methodsFor: 'accessing' stamp: 'mir 12/7/2000 15:34'!
driver
	^driver! !

!SAXHandler methodsFor: 'accessing' stamp: 'mir 12/7/2000 15:34'!
driver: aDriver
	driver := aDriver.
	driver saxHandler: self! !

!SAXHandler methodsFor: 'lexical' stamp: 'JAAyer 2/21/2010 15:13'!
endCData
	"This call corresponds to the Java SAX ext call
	endCData()."! !

!SAXHandler methodsFor: 'content' stamp: 'mir 1/8/2002 18:26'!
endDocument
	"This call corresponds to the Java SAX call
	endDocument()."
	eod := true! !

!SAXHandler methodsFor: 'content' stamp: 'mir 8/14/2000 18:07'!
endElement: elementName
! !

!SAXHandler methodsFor: 'content' stamp: 'mir 6/5/2003 19:05'!
endElement: elementName namespace: namespace namespaceURI: namespaceURI qualifiedName: qualifiedName
	"This call corresponds to the Java SAX call
	endElement(java.lang.String namespaceURI, java.lang.String localName, java.lang.String qName).
	By default this call is mapped to the following more convenient call:"

	self endElement: elementName! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 3/3/2010 23:00'!
endElement: elementName namespace: namespace qualifiedName: qualifiedName scope: namespaceScope
	self
		endElement: elementName
		namespace: namespace
		namespaceURI: (namespaceScope resolveNamespace: namespace)
		qualifiedName: qualifiedName! !

!SAXHandler methodsFor: 'lexical' stamp: 'mir 8/11/2000 18:53'!
endEntity: entityName
	"This call corresponds to the Java SAX ext call
	endEntity(java.lang.String name)."! !

!SAXHandler methodsFor: 'content' stamp: 'mir 8/11/2000 16:25'!
endPrefixMapping: prefix
	"This call corresonds to the Java SAX call
	endPrefixMapping(java.lang.String prefix)."! !

!SAXHandler methodsFor: 'accessing' stamp: 'mir 1/8/2002 18:18'!
eod
	^eod! !

!SAXHandler methodsFor: 'content' stamp: 'mir 8/11/2000 16:25'!
ignorableWhitespace: aString
	"This call corresonds to the Java SAX call
	ignorableWhitespace(char[] ch, int start, int length)."! !

!SAXHandler methodsFor: 'initialize' stamp: 'mir 1/8/2002 18:18'!
initialize
	eod := false! !

!SAXHandler methodsFor: 'parsing' stamp: 'mir 1/8/2002 18:18'!
parseDocument
	[self driver nextEntity isNil or: [self eod]] whileFalse! !

!SAXHandler methodsFor: 'content' stamp: 'mir 8/11/2000 16:26'!
processingInstruction: piName data: dataString
	"This call corresonds to the Java SAX call
	processingInstruction(java.lang.String target, java.lang.String data)."! !

!SAXHandler methodsFor: 'entity' stamp: 'mir 8/11/2000 17:33'!
resolveEntity: publicID systemID: systemID
	"This call corresonds to the Java SAX call
	resolveEntity(java.lang.String publicId, java.lang.String systemId)."! !

!SAXHandler methodsFor: 'content' stamp: 'mir 8/11/2000 16:45'!
skippedEntity: aString
	"This call corresonds to the Java SAX call
	skippedEntity(java.lang.String name)."! !

!SAXHandler methodsFor: 'lexical' stamp: 'mir 8/11/2000 18:53'!
startCData
	"This call corresponds to the Java SAX ext call
	startCData()."! !

!SAXHandler methodsFor: 'lexical' stamp: 'mir 8/11/2000 18:54'!
startDTD: declName publicID: publicID systemID: systemID
	"This call corresponds to the Java SAX ext call
	startDTD(java.lang.String name, java.lang.String publicId, java.lang.String systemId)."! !

!SAXHandler methodsFor: 'content' stamp: 'mir 8/11/2000 16:45'!
startDocument
	"This call corresonds to the Java SAX call
	startDocument()."! !

!SAXHandler methodsFor: 'content' stamp: 'mir 8/14/2000 18:07'!
startElement: elementName attributeList: attributeList
! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 3/3/2010 23:05'!
startElement: localName namespace: namespace attributeList: attributeList scope: namespaceScope
	self
		startElement: localName
		namespace: namespace
		namespaceURI: (namespaceScope resolveNamespace: namespace)
		attributeList: attributeList! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 3/3/2010 23:05'!
startElement: localName namespace: namespace namespaceURI: namespaceUri attributeList: attributeList
	"This call corresonds to the Java SAX call
	startElement(java.lang.String namespaceURI, java.lang.String localName, java.lang.String qName, Attributes atts).
	By default this call is mapped to the following more convenient call:"

	self startElement: localName attributeList: attributeList! !

!SAXHandler methodsFor: 'lexical' stamp: 'mir 8/11/2000 18:54'!
startEntity: entityName
	"This call corresponds to the Java SAX ext call
	startEntity(java.lang.String name)."! !

!SAXHandler methodsFor: 'content' stamp: 'mir 8/11/2000 16:47'!
startPrefixMapping: prefix uri: uri
	"This call corresonds to the Java SAX call
	startPrefixMapping(java.lang.String prefix, java.lang.String uri)."! !

!SAXHandler methodsFor: 'accessing' stamp: 'mir 6/5/2003 19:28'!
useNamespaces: aBoolean
	self driver useNamespaces: aBoolean! !

SAXHandler subclass: #XMLDOMParser
	instanceVariableNames: 'entity stack incremental'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Parser'!

!XMLDOMParser commentStamp: 'JAAyer 2/21/2010 17:56' prior: 0!
XMLDOMParser parses XML into a tree of XML nodes representing the document structure. To see what kinds of nodes it produces and what can be done with them, browse XML-Parser-Nodes.!

!XMLDOMParser class methodsFor: 'instance creation' stamp: 'JAAyer 2/21/2010 00:17'!
parse: aStream
	^ self parseDocumentFrom: aStream! !

!XMLDOMParser class methodsFor: 'instance creation' stamp: 'JAAyer 3/3/2010 22:00'!
parseDocumentFrom: aStream
	^self parseDocumentFrom: aStream useNamespaces: true! !

!XMLDOMParser class methodsFor: 'instance creation' stamp: 'JAAyer 2/13/2010 10:44'!
parseDocumentFrom: aStream useNamespaces: aBoolean
	^(super
		parseDocumentFrom: (aStream isStream ifTrue: [aStream] ifFalse: [aStream readStream])
		useNamespaces: aBoolean) document! !

!XMLDOMParser methodsFor: 'content' stamp: 'JAAyer 2/18/2010 11:54'!
characters: aString
	self top addContent: aString
! !

!XMLDOMParser methodsFor: 'content' stamp: 'thf 9/2/2005 14:09'!
documentAttributes: attributeList
	self document version: (attributeList at: 'version' ifAbsent: [nil]).
	self document encoding: (attributeList at: 'encoding' ifAbsent: [nil]).
	self document requiredMarkup: (attributeList at: 'requiredMarkup' ifAbsent: [nil]).
! !

!XMLDOMParser methodsFor: 'parsing' stamp: 'mir 6/5/2003 19:29'!
domDocument
	[self startDocument; parseDocument]
		ensure: [self driver stream close].
	^document! !

!XMLDOMParser methodsFor: 'content' stamp: 'mir 1/8/2002 18:19'!
endDocument
	self pop.
	super endDocument! !

!XMLDOMParser methodsFor: 'content' stamp: 'mir 1/8/2002 18:11'!
endElement: elementName
	| currentElement |
	currentElement := self pop.
	currentElement name = elementName
		ifFalse: [self driver errorExpected: 'End tag "', elementName , '" doesn''t match "' , currentElement name , '".']! !

!XMLDOMParser methodsFor: 'content' stamp: 'JAAyer 3/3/2010 02:36'!
endElement: localName namespace: namespace namespaceURI: uri qualifiedName: qualifiedName
	| currentElement |
	currentElement := self pop.
	(currentElement namespace isNil)
		ifTrue: [
			currentElement localName = localName asSymbol
				ifFalse: [
					self
						errorExpectedEndTag: currentElement
						localName butGot: localName]]
		ifFalse: [
			currentElement qualifiedName = qualifiedName asSymbol
				ifFalse: [
					self
						errorExpectedEndTag: currentElement qualifiedName
						butGot: qualifiedName]]! !

!XMLDOMParser methodsFor: 'content' stamp: 'JAAyer 3/3/2010 23:00'!
endElement: localName namespace: namespace qualifiedName: qualifiedName scope: namespaceScope
	| currentElement |
	currentElement := self pop.
	(currentElement namespace isNil)
		ifTrue: [
			currentElement localName = localName asSymbol
				ifFalse: [
					self
						errorExpectedEndTag: currentElement
						localName butGot: localName]]
		ifFalse: [
			currentElement qualifiedName = qualifiedName asSymbol
				ifFalse: [
					self
						errorExpectedEndTag: currentElement qualifiedName
						butGot: qualifiedName]]! !

!XMLDOMParser methodsFor: 'errors' stamp: 'JAAyer 3/2/2010 04:19'!
errorExpectedEndTag: anExpectedEndTag butGot: aTag
	self driver
		errorExpected: 'End tag "', aTag, '" doesn''t match "', anExpectedEndTag, '".'! !

!XMLDOMParser methodsFor: 'accessing' stamp: 'mir 1/8/2001 12:05'!
incremental
	^incremental! !

!XMLDOMParser methodsFor: 'accessing' stamp: 'mir 1/8/2001 12:05'!
incremental: aBoolean
	incremental := aBoolean! !

!XMLDOMParser methodsFor: 'initialize' stamp: 'mir 1/8/2001 12:05'!
initialize
	super initialize.
	stack := OrderedCollection new.
	incremental := false! !

!XMLDOMParser methodsFor: 'parsing' stamp: 'mir 6/28/2001 18:45'!
nextEntity
	| currentTop |
	currentTop := self top.
	[self driver nextEntity isNil
		or: [self top ~~ currentTop]] whileTrue.
	^entity! !

!XMLDOMParser methodsFor: 'parsing' stamp: 'mir 12/21/2000 14:02'!
nextEntityStart
	[self driver nextEntity.
	self stack isEmpty] whileTrue.
	^entity! !

!XMLDOMParser methodsFor: 'private' stamp: 'mir 1/8/2001 12:04'!
pop
	| oldTop |
	oldTop := self stack removeLast.
	entity := oldTop.
	^oldTop! !

!XMLDOMParser methodsFor: 'content' stamp: 'JAAyer 2/12/2010 17:30'!
processingInstruction: piName data: dataString
	| piNode |
	piNode := XMLPI target: piName data: dataString.
	self top addNode: piNode! !

!XMLDOMParser methodsFor: 'private' stamp: 'mir 1/8/2001 12:02'!
push: anObject
	self stack add: anObject.
	entity := anObject
! !

!XMLDOMParser methodsFor: 'private' stamp: 'JAAyer 2/12/2010 22:35'!
pushElement: anElement

	(self incremental not and: [self stack notEmpty])
			ifTrue: [self top addElement: anElement].

	self push: anElement.! !

!XMLDOMParser methodsFor: 'private' stamp: 'mir 8/14/2000 18:28'!
stack
	^stack! !

!XMLDOMParser methodsFor: 'content' stamp: 'mir 11/30/2000 18:14'!
startDocument
	self document: XMLDocument new.
	self push: self document ! !

!XMLDOMParser methodsFor: 'content' stamp: 'JAAyer 2/12/2010 22:37'!
startElement: elementName attributeList: attributeList

	self pushElement: (XMLElement named: elementName attributes: attributeList)! !

!XMLDOMParser methodsFor: 'content' stamp: 'JAAyer 3/3/2010 06:42'!
startElement: localName namespace: namespace attributeList: attributeList scope: namespaceScope
	self pushElement:
		(XMLElement
			named: localName
			namespace: namespace
			attributes: attributeList
			scope: namespaceScope)! !

!XMLDOMParser methodsFor: 'private' stamp: 'mir 1/8/2001 11:46'!
top
	^self stack isEmpty
		ifTrue: [nil]
		ifFalse: [self stack last]! !

Object subclass: #XMLNamespaceScope
	instanceVariableNames: 'currentBindings defaultNamespace'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLNamespaceScope commentStamp: 'JAAyer 3/4/2010 00:10' prior: 0!
An XMLNamespaceScope is a set of bindings between namespaces prefixes and URIs, and also a default URI. Scopes can be created from other scopes by sending #enclosingScope: to XMLNamespaceScope with the other XMLNamespaceScope as the argument.!

!XMLNamespaceScope class methodsFor: 'instance creation' stamp: 'JAAyer 3/3/2010 20:48'!
enclosingScope: aScope
	^ self new inheritBindingsFromScope: aScope! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 3/3/2010 20:58'!
aliasesFor: namespace
	"Locate all namespaces that are aliases of the given URI."
	| uri |

	uri := self
		namespaceURIOf: namespace
		ifAbsent: [self errorUndefinedAttributeNamespace: namespace asString].

	^ (self boundNamespaces select: [:ns |
		(self isNamespace: ns boundTo: uri)
			and: [ns ~= namespace]]) asSet! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 3/3/2010 02:24'!
bindNamespace: aNamespace to: aUri
	((aNamespace = 'xmlns') or: [aNamespace isEmptyOrNil])
		ifTrue: [self defaultNamespace: aUri]
		ifFalse: [self currentBindings at: aNamespace put: aUri]! !

!XMLNamespaceScope methodsFor: 'enumerating' stamp: 'JAAyer 3/3/2010 07:44'!
bindingsDo: aTwoArgumentBlock
	self boundNamespaces do: [:namespace |
		aTwoArgumentBlock
			value: namespace
			value: (self resolveNamespace: namespace)]! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 3/3/2010 08:04'!
boundNamespaces
	^ self currentBindings keys asSortedCollection! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 3/2/2010 20:35'!
currentBindings
	^ currentBindings ifNil: [currentBindings := Dictionary new]! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 3/2/2010 20:54'!
defaultNamespace
	^ defaultNamespace! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 3/2/2010 20:45'!
defaultNamespace: aNamespaceUri
	defaultNamespace := aNamespaceUri! !

!XMLNamespaceScope methodsFor: 'errors' stamp: 'JAAyer 3/4/2010 00:31'!
errorAttribute: anAttribute isAliasFor: anAlias withURI: aNamespaceUri
	self error: ('Attributes ', anAttribute, ' and ',
		anAlias, ' are aliased to namespace ', aNamespaceUri)! !

!XMLNamespaceScope methodsFor: 'errors' stamp: 'JAAyer 3/2/2010 21:44'!
errorUndefinedAttributeNamespace: aNamespace
	self error: 'Attribute refers to undefined namespace ', aNamespace! !

!XMLNamespaceScope methodsFor: 'initializing' stamp: 'JAAyer 3/3/2010 20:47'!
inheritBindingsFromScope: anEnclosingScope.
	self defaultNamespace
		ifNil: [self defaultNamespace: anEnclosingScope defaultNamespace].

	anEnclosingScope bindingsDo: [:prefix :uri |
		(self isBoundNamespace: prefix)
			ifFalse: [self bindNamespace: prefix to: uri]].! !

!XMLNamespaceScope methodsFor: 'testing' stamp: 'JAAyer 3/3/2010 01:42'!
isBoundNamespace: aNamespace
	^ self currentBindings includesKey: aNamespace! !

!XMLNamespaceScope methodsFor: 'testing' stamp: 'JAAyer 3/3/2010 06:14'!
isNamespace: aNamespace boundTo: aUri
	^ ((self isBoundNamespace: aNamespace)
		and: [(self resolveNamespace: aNamespace) = aUri])! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 3/3/2010 06:06'!
resolveNamespace: aNamespace
	"Retrieve the URI of the given namespace prefix, if it is defined. A nil namespace
	returns the global namespace"

	^ self resolveNamespace: aNamespace ifUnbound: [nil]! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 3/3/2010 06:07'!
resolveNamespace: aNamespace ifUnbound: aBlock
	"Retrieve the URI of the given namespace prefix, if it is defined. 
	A nil namespace returns the default namespace. 
	If no namespace can be found the value of the block is returned"

	^ (aNamespace isEmptyOrNil or: [aNamespace = 'xmlns'])
		ifTrue: [self defaultNamespace]
		ifFalse: [self currentBindings at: aNamespace ifAbsent: aBlock]! !

!XMLNamespaceScope methodsFor: 'validation' stamp: 'JAAyer 3/4/2010 00:31'!
validateAttributes: attributeList
	"check all attribute namespaces are defined and not duplicated by aliasing"
	| namespace localName qualifiedAlias |

	attributeList keysDo: [:attrName |
		attrName splitQualifiedNameInto: [:ns :ln |
			namespace := ns.
			localName := ln].
		namespace ifNotNil: [
			(self currentScope aliasesFor: namespace) do: [:alias |
				qualifiedAlias := alias , ':' , localName.
				(attributeList includesKey: qualifiedAlias)
					ifTrue: [
						self
							errorAttribute: attrName
							isAliasFor: qualifiedAlias
							withURI: (self resolveNamespace: namespace)]]]]! !

Object subclass: #XMLNamespaceScopeStack
	instanceVariableNames: 'scopes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLNamespaceScopeStack commentStamp: 'JAAyer 3/4/2010 00:14' prior: 0!
A XMLNamespaceScopeStack keeps track of XMLNamespaceScopes. Sending #enterScope will cause an instance to push a new scope onto the stack that inherits all bindings from the current enclosing scope. Sending #leaveScop causes that scope to be removed.!

!XMLNamespaceScopeStack methodsFor: 'accessing' stamp: 'JAAyer 3/3/2010 22:33'!
currentLevel
	^ self scopes size! !

!XMLNamespaceScopeStack methodsFor: 'accessing' stamp: 'JAAyer 3/3/2010 22:34'!
currentScope
	^ self scopes
		at: self currentLevel
		ifAbsent: [self emptyScope]! !

!XMLNamespaceScopeStack methodsFor: 'scope' stamp: 'JAAyer 3/3/2010 22:34'!
declareNamespace: aNamespace uri: aUri
	"Declare the given name space prefix with the given URL"

	self currentScope bindNamespace: aNamespace to: aUri! !

!XMLNamespaceScopeStack methodsFor: 'accessing' stamp: 'JAAyer 3/3/2010 22:37'!
defaultNamespace
	^ self currentScope defaultNamespace! !

!XMLNamespaceScopeStack methodsFor: 'private' stamp: 'JAAyer 3/2/2010 22:14'!
emptyScope
	^ XMLNamespaceScope new! !

!XMLNamespaceScopeStack methodsFor: 'accessing' stamp: 'JAAyer 3/2/2010 22:14'!
enclosingScope
	^ self scopes
		at: self scopes size - 1
		ifAbsent: [self emptyScope]! !

!XMLNamespaceScopeStack methodsFor: 'scope' stamp: 'JAAyer 3/4/2010 00:12'!
enterScope
	self pushScope: (XMLNamespaceScope enclosingScope: self currentScope)! !

!XMLNamespaceScopeStack methodsFor: 'testing' stamp: 'JAAyer 3/3/2010 22:38'!
isBoundNamespace: aNamespace
	^ self currentScope isBoundNamespace: aNamespace! !

!XMLNamespaceScopeStack methodsFor: 'testing' stamp: 'JAAyer 3/3/2010 22:36'!
isNamespace: aNamespace boundTo: aUri
	^ self currentScope isNamespace: aNamespace boundTo: aUri! !

!XMLNamespaceScopeStack methodsFor: 'scope' stamp: 'JAAyer 3/4/2010 00:12'!
leaveScope
	self popScope! !

!XMLNamespaceScopeStack methodsFor: 'accessing' stamp: 'JAAyer 3/4/2010 00:12'!
popScope
	self scopes removeLast! !

!XMLNamespaceScopeStack methodsFor: 'accessing' stamp: 'JAAyer 3/4/2010 00:11'!
pushScope: aScope
	self scopes add: aScope! !

!XMLNamespaceScopeStack methodsFor: 'accessing' stamp: 'JAAyer 3/3/2010 22:37'!
resolveNamespace: aNamespace
	^ self currentScope resolveNamespace: aNamespace! !

!XMLNamespaceScopeStack methodsFor: 'accessing' stamp: 'JAAyer 3/3/2010 22:37'!
resolveNamespace: aNamespace ifUnbound: aBlock
	^ self currentScope resolveNamespace: aNamespace ifUnbound: aBlock! !

!XMLNamespaceScopeStack methodsFor: 'accessing' stamp: 'JAAyer 3/2/2010 21:35'!
scopes
	^ scopes ifNil: [scopes := OrderedCollection new: 20]! !

!XMLNamespaceScopeStack methodsFor: 'validation' stamp: 'JAAyer 3/4/2010 00:31'!
validateAttributes: attributeList
	self currentScope validateAttributes: attributeList! !

Object subclass: #XMLNode
	instanceVariableNames: 'parent'
	classVariableNames: 'CanonicalTable'
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLNode commentStamp: 'JAAyer 2/19/2010 03:30' prior: 0!
A XMLNode is xxxxxxxxx.

Instance Variables
	parent:		<Object>

parent
	- xxxxx
!

!XMLNode methodsFor: 'accessing' stamp: 'mir 1/8/2002 18:44'!
addContent: contentString
	SAXParseException signal: 'Illegal string data.'! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 2/25/2010 14:37'!
hasParent
	^ self parent notNil! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 2/19/2010 23:50'!
isDocument
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 2/12/2010 20:23'!
isElement
	^ self isTag! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 2/21/2010 19:50'!
isEmpty
	^ true! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 2/19/2010 15:57'!
isPI
	^ self isProcessingInstruction! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 2/25/2010 01:48'!
isProcessingInstruction
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 2/25/2010 01:48'!
isTag
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 2/21/2010 19:50'!
isTagOrText
	^ self isTag or: [self isText]! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 2/25/2010 01:48'!
isText
	^ false! !

!XMLNode methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
nextNode
	^ self hasParent ifTrue: [self parent nodeAfter: self]! !

!XMLNode methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
parent
	^ parent! !

!XMLNode methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
parent: aParentNode
	parent := aParentNode! !

!XMLNode methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
previousNode
	^ self hasParent ifTrue: [self parent nodeBefore: self]! !

!XMLNode methodsFor: 'printing' stamp: 'mir 1/17/2002 15:45'!
printOn: stream
	self printXMLOn: (XMLWriter on: stream)! !

!XMLNode methodsFor: 'printing' stamp: 'mir 1/17/2002 15:45'!
printXMLOn: writer
	self subclassResponsibility! !

XMLNode subclass: #XMLNodeWithElements
	instanceVariableNames: 'nodes elements elementsByName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLNodeWithElements commentStamp: 'JAAyer 3/1/2010 18:53' prior: 0!
This class represents a node with child nodes. You will likely use its API through either XMLDocument or XMLElement. To retrieve child elements by name, you can send any of the #element- messages in "accessing." The #firstTag- messages search the receiver and its descendants using depth-first traversal for specific elements, and the #tagsNamed- messages perform some sort of iteration over descendent elements also using depth-first traversal.!

XMLNodeWithElements subclass: #XMLDocument
	instanceVariableNames: 'dtd version encoding requiredMarkup'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 01:48'!
dtd
	^ dtd! !

!XMLDocument methodsFor: 'accessing' stamp: 'mir 11/30/2000 17:48'!
dtd: aDTD
	dtd := aDTD! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 01:48'!
encoding	
	^ encoding ifNil: ['UTF-8']! !

!XMLDocument methodsFor: 'accessing' stamp: 'mir 1/17/2002 12:57'!
encoding: aString	
	encoding := aString! !

!XMLDocument methodsFor: 'testing' stamp: 'JAAyer 2/19/2010 23:51'!
isDocument
	^ true! !

!XMLDocument methodsFor: 'printing' stamp: 'JAAyer 2/25/2010 14:37'!
printCanonicalOn: aStream
	| writer |
	writer := XMLWriter on: aStream.
	writer canonical: true.
	self printXMLOn: writer! !

!XMLDocument methodsFor: 'printing' stamp: 'mir 5/16/2003 14:08'!
printXMLOn: writer
	version ifNotNil: [writer xmlDeclaration: self version encoding: self encoding].
	super printXMLOn: writer! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 01:48'!
requiredMarkup	
	^ requiredMarkup! !

!XMLDocument methodsFor: 'accessing' stamp: 'mir 1/17/2002 12:57'!
requiredMarkup: aString	
	requiredMarkup := aString! !

!XMLDocument methodsFor: 'accessing' stamp: 'mir 11/29/2007 14:42'!
root
	"return my root element"
	^ self topElement ! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 01:48'!
version	
	^ version! !

!XMLDocument methodsFor: 'accessing' stamp: 'mir 1/17/2002 12:57'!
version: aString	
	version := aString! !

XMLNodeWithElements subclass: #XMLElement
	instanceVariableNames: 'name localName attributes namespace namespaceScope'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 2/25/2010 01:48'!
named: aString
	^ self new name: aString! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 2/25/2010 01:48'!
named: aString attributes: attributeList
	^ self new
		name: aString;
		setAttributes: attributeList! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 2/25/2010 01:48'!
named: aString attributes: attributeList contentString: aContentString
	^ self new
		name: aString;
		setAttributes: attributeList;
		contentString: aContentString! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 2/25/2010 01:48'!
named: aString contentString: aContentString
	^ self new
		name: aString;
		contentString: aContentString! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 3/3/2010 06:59'!
named: aString namespace: ns attributes: attributeList scope: namespaceScope
	^ self new
		setScope: namespaceScope;
		setAttributes: attributeList;
		name: aString namespace: ns! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 2/25/2010 01:48'!
named: aString namespace: ns uri: uri attributes: attributeList
	^ self new
		name: aString namespace: ns uri: uri;
		setAttributes: attributeList! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 3/1/2010 01:38'!
@ aSymbolOrString
	"shorthand form"
	^ self at: aSymbolOrString! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 3/1/2010 19:04'!
addContent: aStringOrStringNode
	"Adds a string or string node. If the last child node is already a string node,
	then contentString gets concatenated to it."

	(self hasContents and: [self nodes last isText])
		ifTrue: [self nodes last addContent: aStringOrStringNode]
		ifFalse: [self addNode: (XMLStringNode string: aStringOrStringNode)]! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 3/2/2010 20:04'!
allAttributeAssociations
	^ self attributes associations asSortedCollection asOrderedCollection! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 3/2/2010 20:03'!
allAttributes
	^ self attributes keys asSortedCollection asOrderedCollection! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 3/1/2010 01:37'!
at: aSymbol
    ^ self at: aSymbol ifAbsent: ['']
! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 3/1/2010 01:37'!
at: aSymbol ifAbsent: aBlock
    ^ self attributeAt: aSymbol ifAbsent: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 3/1/2010 01:37'!
at: aSymbol put: aValue
    ^ self attributeAt: aSymbol put: aValue! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 3/1/2010 01:36'!
attributeAt: aSymbolOrString
	^self attributeAt: aSymbolOrString ifAbsent: [nil]! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 3/1/2010 01:36'!
attributeAt: aSymbolOrString ifAbsent: aBlock
	^self attributes at: aSymbolOrString asSymbol ifAbsent: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 3/1/2010 01:37'!
attributeAt: aSymbol put: aValue
	^ self attributes at: aSymbol asSymbol put: aValue! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 2/24/2010 21:09'!
attributes
	^ attributes ifNil: [attributes := IdentityDictionary new]! !

!XMLElement methodsFor: 'private' stamp: 'JAAyer 3/3/2010 21:58'!
bindNamespace: aPrefix to: aUri
	| oldBinding |

	oldBinding := self namespaceScope
		resolveNamespace: aPrefix
		ifUnbound: [nil].
	self
		updateEnclosedScopesWith: [:scope |
			scope bindNamespace: aPrefix to: aUri]
		where: [:scope |
			(scope resolveNamespace: aPrefix ifUnbound: [nil]) = oldBinding]! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 01:48'!
characterData
	^ self contentString! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 2/24/2010 21:32'!
contentString
	| firstContentNode |

	firstContentNode := self elementsAndContents at: 1 ifAbsent: [^ ''].
	^ firstContentNode isText
		ifTrue: [firstContentNode string]
		ifFalse: ['']! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
contentString: aString
	self
		removeNodes;
		addContent: aString! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 01:48'!
contentStringAt: aSymbol
	^ (self elementAt: aSymbol ifAbsent: [^ '']) contentString! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
contents
	^ self nodes select: [:each | each isText]! !

!XMLElement methodsFor: 'enumerating' stamp: 'JAAyer 2/12/2010 19:09'!
contentsDo: aBlock
	self nodesDo: [:each | each isText ifTrue: [aBlock value: each]]! !

!XMLElement methodsFor: 'namespace' stamp: 'JAAyer 3/3/2010 06:19'!
defaultNamespace
	^ self namespaceScope defaultNamespace! !

!XMLElement methodsFor: 'namespace' stamp: 'JAAyer 3/3/2010 21:58'!
defaultNamespace: aNamespaceUri
	| oldDefaultNamespace |

	oldDefaultNamespace := self defaultNamespace.
	self
		updateEnclosedScopesWith: [:scope |
			scope defaultNamespace: aNamespaceUri]
		where: [:scope |
			scope defaultNamespace = oldDefaultNamespace].! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 2/15/2010 12:22'!
elementsAndContents
	^ self nodes select: [:each | each isTagOrText]! !

!XMLElement methodsFor: 'enumerating' stamp: 'JAAyer 2/15/2010 12:23'!
elementsAndContentsDo: aBlock
	self nodesDo: [:each | each isTagOrText ifTrue: [aBlock value: each]]! !

!XMLElement methodsFor: 'errors' stamp: 'JAAyer 3/3/2010 02:15'!
errorUnboundNamespace: aPrefix
	self error: 'Unbound namespace "', aPrefix, '"'! !

!XMLElement methodsFor: 'namespace' stamp: 'JAAyer 3/3/2010 00:05'!
expandedName
	"Returns a string containing the local name expanded with its namespace URI"
	^ self namespaceURI
		ifNil: [self localName asString]
		ifNotNil: ['{', self namespaceURI, '}', self localName asString]! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 2/25/2010 14:37'!
hasAttributes
	^ self attributes notEmpty! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 2/25/2010 14:37'!
hasContents
	^ self contents notEmpty! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 3/1/2010 02:44'!
hasPrefix
	^ self localName ~~ self qualifiedName! !

!XMLElement methodsFor: 'searching' stamp: 'JAAyer 2/14/2010 21:30'!
ifNamed: aSymbol do: aOneArgumentBlock
	"If the receiver tag equals aSymbol, evaluate aOneArgumentBlock with the receiver"

	(self isNamed: aSymbol)
		ifTrue: [aOneArgumentBlock valueWithPossibleArgument: self]
! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 2/25/2010 14:37'!
includesAttribute: aSymbol
	^ self attributes includesKey: aSymbol asSymbol! !

!XMLElement methodsFor: 'private' stamp: 'JAAyer 3/3/2010 21:21'!
inheritBindingsFromScope: aScope
	self recurse: [:each |
		each namespaceScope inheritBindingsFromScope: aScope]! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 3/1/2010 02:45'!
isNamed: aSymbol
	^ (self localName == aSymbol asSymbol)
		or: [self qualifiedName == aSymbol asSymbol]! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 2/26/2010 22:10'!
isNamedAny: aSymbolArray
	^ aSymbolArray anySatisfy: [:each | self isNamed: each]! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 2/25/2010 01:48'!
isTag
	^ true! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 06:01'!
localName
	^ localName ifNil: [#'']! !

!XMLElement methodsFor: 'accessing' stamp: 'sd 5/25/2003 18:25'!
name
	^ self qualifiedName! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 3/3/2010 21:17'!
name: aLocalName
	name :=
		(self namespace
			ifNil: [aLocalName]
			ifNotNil: [self namespace asString, ':', aLocalName asString]) asSymbol.
	localName := aLocalName asSymbol.

	self notifyParentOfChange.! !

!XMLElement methodsFor: 'namespace' stamp: 'JAAyer 3/3/2010 07:18'!
name: aLocalName namespace: aPrefix
	self
		name: aLocalName
		namespace: aPrefix
		uri: nil! !

!XMLElement methodsFor: 'namespace' stamp: 'JAAyer 3/3/2010 21:38'!
name: aLocalName namespace: aPrefix uri: aUri
	(aPrefix isEmptyOrNil or: [aPrefix = 'xmlns'])
		ifTrue: [
			aUri ifNotNil: [self defaultNamespace: aUri].
			namespace := nil]
		ifFalse: [
			aUri
				ifNotNil: [self bindNamespace: aPrefix to: aUri]
				ifNil: [
					(self namespaceScope isBoundNamespace: aPrefix)
						ifFalse: [self errorUnboundNamespace: aPrefix]].
			namespace := aPrefix].

	self name: aLocalName.! !

!XMLElement methodsFor: 'namespace' stamp: 'JAAyer 3/2/2010 23:46'!
namespace
	^ namespace! !

!XMLElement methodsFor: 'namespace' stamp: 'JAAyer 3/3/2010 07:17'!
namespace: aPrefix
	self
		name: self localName
		namespace: aPrefix! !

!XMLElement methodsFor: 'namespace' stamp: 'JAAyer 2/25/2010 14:37'!
namespace: aPrefix uri: aUri
	self
		name: self localName
		namespace: aPrefix
		uri: aUri! !

!XMLElement methodsFor: 'namespace' stamp: 'JAAyer 3/2/2010 23:45'!
namespaceScope
	^ namespaceScope ifNil: [namespaceScope := XMLNamespaceScope new]! !

!XMLElement methodsFor: 'namespace' stamp: 'JAAyer 3/3/2010 06:18'!
namespaceURI
	^ self namespaceScope resolveNamespace: self namespace! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
nextElement
	^ self hasParent ifTrue: [self parent elementAfter: self]! !

!XMLElement methodsFor: 'private' stamp: 'JAAyer 2/28/2010 19:44'!
notifyParentOfChange
	self hasParent ifTrue: [self parent changedChildElement: self]! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
previousElement
	^ self hasParent ifTrue: [self parent elementBefore: self]! !

!XMLElement methodsFor: 'printing' stamp: 'JAAyer 3/3/2010 07:31'!
printXMLOn: writer
	writer
		startElement: self name
		attributeList: self attributes
		scope: self namespaceScope.
	(writer canonical not
		and: [self isEmpty and: [self attributes isEmpty not]])
		ifTrue: [writer endEmptyTag: self name]
		ifFalse: [
			writer endTag.
			super printXMLOn: writer.
			writer endTag: self name]! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 06:02'!
qualifiedName
	^ name ifNil: [#'']! !

!XMLElement methodsFor: 'searching' stamp: 'JAAyer 2/26/2010 23:05'!
recurse: aBlock
	"See superclass version."

	aBlock valueWithPossibleArgument: self.

	super recurse: aBlock.! !

!XMLElement methodsFor: 'searching' stamp: 'JAAyer 2/26/2010 23:05'!
recurseUntilTrue: aBlock
	"See superclass version."

	(aBlock valueWithPossibleArgument: self)
		ifTrue: [^ self].

	^ super recurseUntilTrue: aBlock.! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 3/1/2010 01:13'!
removeAttribute: aSymbol
	^ self attributes removeKey: aSymbol asSymbol! !

!XMLElement methodsFor: 'initialize' stamp: 'JAAyer 3/3/2010 07:58'!
setAttributes: aDictionary
	self attributes removeAll.
	aDictionary ifNil: [^ self].
	aDictionary keysAndValuesDo: [:key :value | self at: key put: value].! !

!XMLElement methodsFor: 'initialize' stamp: 'JAAyer 3/2/2010 23:44'!
setScope: aNamespaceScope
	namespaceScope := XMLNamespaceScope enclosingScope: aNamespaceScope! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 06:02'!
tag
	^ self name! !

!XMLElement methodsFor: 'private' stamp: 'JAAyer 3/4/2010 00:48'!
updateEnclosedScopesWith: aBlock where: aConditional
	"Updates the receiver's scope and the scopes of all descendants with aBlock
	where aConditional is true. It stops descending down a path of a subtree
	as soon as aConditional is false, but continues down other paths."

	(aConditional value: self namespaceScope)
		ifTrue: [aBlock value: self namespaceScope]
		ifFalse: [^ self].

	self elementsDo: [:each |
		each updateEnclosedScopesWith: aBlock where: aConditional]! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
valueFor: aSymbol
	self deprecated: 'use #at: as a shortcut instead'.

	^ self at: aSymbol.! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
valueFor: aSymbol ifAbsent: aBlock 
	self deprecated: 'use #at:ifAbsent: as a shortcut instead'.

	^ self at: aSymbol ifAbsent: aBlock! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
addElement: anElement
	self addNode: anElement! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
addNode: aNode
	^ self with: [self nodes add: aNode] add: aNode.! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
addNode: aNode after: afterNode
	^ self with: [self nodes add: aNode after: afterNode] add: aNode! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
addNode: aNode before: beforeNode
	^ self with: [self nodes add: aNode before: beforeNode] add: aNode.! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
addNodeFirst: aNode
	^ self isEmpty
		ifTrue: [self addNode: aNode]
		ifFalse: [self addNode: aNode before: self topNode]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
addNodes: aNodeCollection
	aNodeCollection do: [:each | self addNode: each].
	^ aNodeCollection.! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'JAAyer 2/25/2010 14:37'!
changedChildElement: anElement
	self rebuildElementCache! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
elementAfter: anElement
	^ self elements after: anElement ifAbsent: [nil]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 01:48'!
elementAt: aSymbol
	^ self elementAt: aSymbol ifAbsent: [nil]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 2/18/2010 20:43'!
elementAt: aSymbol ifAbsent: aBlock
	^ (self elementsAt: aSymbol ifAbsent: [^ aBlock value]) first! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 2/19/2010 02:21'!
elementAtAny: aSymbolArray
	^ self elementAtAny: aSymbolArray ifAbsent: [nil]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 2/19/2010 02:21'!
elementAtAny: aSymbolArray ifAbsent: aBlock
	| answer |

	aSymbolArray do: [:each | (answer := self elementAt: each) ifNotNil: [^ answer]].
	^ aBlock value.! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
elementBefore: anElement
	^ self elements before: anElement ifAbsent: [nil]! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'JAAyer 3/1/2010 02:44'!
elementCacheAdd: anElement
	self elements add: anElement.

	self elementCacheAt: anElement localName add: anElement.
	anElement hasPrefix
		ifTrue: [self elementCacheAt: anElement qualifiedName add: anElement].! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'JAAyer 2/25/2010 14:37'!
elementCacheAt: aSymbol add: anElement
	(self elementsByName
		at: aSymbol asSymbol
		ifAbsentPut: [self newElements]) add: anElement! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'JAAyer 3/1/2010 02:58'!
elementCacheAt: aSymbol remove: anElement
	(self elementsByName at: aSymbol asSymbol) remove: anElement.

	(self elementsAt: aSymbol)
		ifEmpty: [self elementsByName removeKey: aSymbol asSymbol].! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'JAAyer 3/1/2010 02:55'!
elementCacheRemove: anElement
	self elements remove: anElement.

	self elementCacheAt: anElement localName remove: anElement.
	anElement hasPrefix
		ifTrue: [self elementCacheAt: anElement qualifiedName remove: anElement].! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 02:04'!
elementUnqualifiedAt: aSymbol
	self deprecated: 'use #elementAt: with an unqualified name instead'.

	^ self elementAt: aSymbol.! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 02:04'!
elementUnqualifiedAt: aSymbol ifAbsent: aBlock
	self deprecated: 'use #elementAt:ifAbsent: with an unqualified name instead'.

	^ self elementAt: aSymbol ifAbsent: aBlock.! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 2/18/2010 00:07'!
elements
	^ elements ifNil: [elements := self newElements]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
elementsAt: aSymbol
	^ self elementsAt: aSymbol ifAbsent: [self newElements]! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 2/27/2010 15:13'!
elementsAt: aSymbol do: aBlock
	(self elementsAt: aSymbol) do: aBlock! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
elementsAt: aSymbol ifAbsent: aBlock
	^ self elementsByName at: aSymbol asSymbol ifAbsent: aBlock.! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'JAAyer 2/25/2010 14:37'!
elementsByName 
	^ elementsByName ifNil: [elementsByName := IdentityDictionary new].! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 2/12/2010 20:57'!
elementsDo: aBlock
	self elements do: aBlock! !

!XMLNodeWithElements methodsFor: 'comparing' stamp: 'JAAyer 2/25/2010 14:37'!
equalTo: aNodeWithElements
	^ (aNodeWithElements respondsTo: #asString)
		and: [self asString = aNodeWithElements asString]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'JAAyer 2/26/2010 22:11'!
firstTagNamed: aSymbol 
	"Return the first encountered node with the specified tag. Pass the message on"

	^ self recurseUntilTrue: [:each | each isNamed: aSymbol]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'JAAyer 2/26/2010 22:11'!
firstTagNamed: aSymbol with: aBlock
	"Return the first encountered node with the specified tag that
	allows the block to evaluate to true. Pass the message on"

	^ self recurseUntilTrue: [:each |
		(each isNamed: aSymbol) and: [aBlock valueWithPossibleArgument: each]]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'JAAyer 2/26/2010 22:10'!
firstTagNamedAny: aSymbolArray
	"Return the first encountered node with any of the specified tag names. Pass the message on"

	^ self recurseUntilTrue: [:each | each isNamedAny: aSymbolArray]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'JAAyer 2/26/2010 22:11'!
firstTagNamedAny: aSymbolArray with: aBlock
	"Return the first encountered node with any of the specified tag names that
	allows the block to evaluate to true. Pass the message on"

	^ self recurseUntilTrue: [:each |
		(each isNamedAny: aSymbolArray) and: [aBlock valueWithPossibleArgument: each]]! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'JAAyer 2/25/2010 01:48'!
hasElements
	^ self elements notEmpty! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'JAAyer 3/1/2010 02:51'!
includesElement: aSymbol
	^ (self elementAt: aSymbol) notNil! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'JAAyer 2/25/2010 14:37'!
includesNode: aNode
	^ self nodes includes: aNode! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'JAAyer 2/25/2010 14:37'!
isEmpty
	^ self nodes isEmpty! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'JAAyer 2/25/2010 14:37'!
newElements
	^ OrderedCollection new! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
nodeAfter: aNode
	^ self nodes after: aNode ifAbsent: [nil]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
nodeBefore: aNode
	^ self nodes before: aNode ifAbsent: [nil]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 2/23/2010 19:00'!
nodes
	^ nodes ifNil: [nodes := OrderedCollection new]! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 2/12/2010 21:12'!
nodesDo: aBlock
	self nodes do: aBlock! !

!XMLNodeWithElements methodsFor: 'printing' stamp: 'JAAyer 2/15/2010 11:38'!
printXMLOn: writer
	self nodesDo: [:each | each printXMLOn: writer].! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'JAAyer 2/27/2010 15:22'!
rebuildElementCache
	"This method rebuilds the #elements and #elementsByName caches using #nodes. It is
	sent by #with:add: when an element node is added to #nodes anywhere except at the
	end and also by changedChildElement: when the name or namespace information of a
	child element is changed.
	
	See #with:add: and #with:remove:"

	self elements removeAll.
	self elementsByName removeAll.

	self nodesDo: [:each | each isTag ifTrue: [self elementCacheAdd: each]].! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'JAAyer 2/27/2010 19:28'!
recurse: aBlock
	"Descend depth-first visiting each element with aBlock."

	self elementsDo: [:each | each recurse: aBlock]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'JAAyer 2/27/2010 19:29'!
recurseUntilTrue: aBlock
	"Descend depth-first visiting each element with aBlock until one such evaluation
	of aBlock with an element is true, then return that element. If no evaluation is
	true, then return nil."

	| result |

	self elementsDo: [:each | (result := each recurseUntilTrue: aBlock) ifNotNil: [^ result]].
	^ nil.! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 3/1/2010 02:55'!
removeNode: aNode
	^ self with: [self nodes remove: aNode] remove: aNode! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
removeNodes
	^ self removeNodes: self nodes copy! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
removeNodes: aNodeCollection
	aNodeCollection do: [:each | self removeNode: each].
	^ aNodeCollection.! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'JAAyer 2/27/2010 15:14'!
tagsNamed: aSymbol childrenDo: aOneArgumentBlock
	"Evaluate aOneArgumentBlock for all children who match"

	self deprecated: 'use #elementsAt:do: instead'.

	self elementsAt: aSymbol do: aOneArgumentBlock.! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'JAAyer 2/13/2010 22:29'!
tagsNamed: aSymbol childrenDoAndRecurse: aOneArgumentBlock
	"Evaluate aOneArgumentBlock for all children who match and recurse"

	self elementsDo: [:each | each tagsNamed: aSymbol do: aOneArgumentBlock]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'JAAyer 2/27/2010 15:14'!
tagsNamed: aSymbol contentsDo: aBlock
	"Evaluate aBlock for all of the contents of the receiver.
	The receiver has no tag, so pass the message on"

	self tagsNamed: aSymbol do: [:each | each contentsDo: aBlock]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'JAAyer 2/26/2010 22:02'!
tagsNamed: aSymbol do: aOneArgumentBlock
	"Search for nodes with tag aSymbol. When encountered evaluate aOneArgumentBlock"

	self recurse: [:each | each ifNamed: aSymbol do: aOneArgumentBlock]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'JAAyer 2/25/2010 14:37'!
tagsNamed: aSymbol ifReceiverDoAndRecurse: aOneArgumentBlock
	self deprecated: 'use #tagsNamed:do: instead'.

	self tagsNamed: aSymbol do: aOneArgumentBlock.! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 2/23/2010 18:57'!
topElement
	^ self elements at: 1 ifAbsent: [nil]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 2/23/2010 18:57'!
topNode
	^ self nodes at: 1 ifAbsent: [nil]! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'JAAyer 3/3/2010 20:46'!
with: aBlock add: aNode
	"This is an execute-around method that takes a block purporting to add aNode
	to #nodes. It evaluates the block, establishes a parent-child relationship,
	and if the node is an XMLElement, adds it to the element cache or rebuilds
	the cache if needed.
	
	Methods adding anything to #nodes should be implemented around this
	message."

	aNode hasParent
		ifTrue: [self error: 'attempt to add node with parent'. ^ nil].

	self with: aBlock onError: [^ nil].
	aNode parent: self.
	aNode isTag ifTrue: [
		self isTag
			ifTrue: [aNode inheritBindingsFromScope: self namespaceScope].

		(self nodes last = aNode)
			ifTrue: [self elementCacheAdd: aNode]
			ifFalse: [self rebuildElementCache]].
	^ aNode.! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'JAAyer 2/25/2010 14:37'!
with: aBlock onError: errorBlock
	^ aBlock
		on: Error
		do: [:error | error signal. errorBlock valueWithPossibleArgument: error]! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'JAAyer 2/25/2010 14:37'!
with: aBlock remove: aNode
	"This is an execute-around method that takes a block purporting to remove
	aNode from #nodes, evaluates it, breaks the parent-child relationship,
	and removes aNode from the element cache if it happens to be an XMLElement.
	
	Methods that remove anything from #nodes should be implemented around this
	message."

	self with: aBlock onError: [^ nil].
	aNode parent: nil.
	aNode isTag ifTrue: [self elementCacheRemove: aNode].
	^ aNode.! !

XMLNode subclass: #XMLPI
	instanceVariableNames: 'target data'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLPI class methodsFor: 'instance creation' stamp: 'JAAyer 2/25/2010 01:48'!
target: targetName data: aString
	^ self new
		target: targetName;
		data: aString! !

!XMLPI methodsFor: 'accessing' stamp: 'JAAyer 2/23/2010 22:40'!
data
	^ data ifNil: ['']! !

!XMLPI methodsFor: 'accessing' stamp: 'mir 1/17/2002 13:02'!
data: aString
	data := aString! !

!XMLPI methodsFor: 'testing' stamp: 'JAAyer 2/25/2010 01:48'!
isProcessingInstruction
	^ true! !

!XMLPI methodsFor: 'printing' stamp: 'mir 1/17/2002 15:53'!
printXMLOn: writer
	writer pi: self target data: self data! !

!XMLPI methodsFor: 'accessing' stamp: 'JAAyer 2/23/2010 22:40'!
target
	^ target ifNil: ['']! !

!XMLPI methodsFor: 'accessing' stamp: 'mir 1/17/2002 13:02'!
target: aString
	target := aString! !

XMLNode subclass: #XMLStringNode
	instanceVariableNames: 'string'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLStringNode classSide methodsFor: 'instance creation' stamp: 'JAAyer 2/25/2010 01:48'!
string: aString
	^ self new string: aString! !

!XMLStringNode methodsFor: 'accessing' stamp: 'JAAyer 2/23/2010 02:55'!
addContent: aString
	self string: self string, (self toString: aString)! !

!XMLStringNode methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 01:48'!
characterData
	^ self string! !

!XMLStringNode methodsFor: 'testing' stamp: 'JAAyer 2/25/2010 01:48'!
isText
	^ true! !

!XMLStringNode methodsFor: 'printing' stamp: 'mir 1/17/2002 15:53'!
printXMLOn: writer
	writer pcData: self string! !

!XMLStringNode methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 01:48'!
string
	^ string ifNil: ['']! !

!XMLStringNode methodsFor: 'accessing' stamp: 'JAAyer 2/23/2010 02:55'!
string: aString
	string := self toString: aString! !

!XMLStringNode methodsFor: 'private' stamp: 'JAAyer 2/23/2010 03:03'!
toString: aString
	^ (aString respondsTo: #string) ifTrue: [aString string] ifFalse: [aString]! !

Object subclass: #XMLTokenizer
	instanceVariableNames: 'stream nestedStreams entities externalEntities parameterEntities parsingMarkup peekChar validating nameBuffer attributeBuffer'
	classVariableNames: 'CharEscapes DigitTable LiteralChars NameDelimiters SeparatorTable'
	poolDictionaries: ''
	category: 'XML-Parser-Parser'!

!XMLTokenizer commentStamp: '<historical>' prior: 0!
XMLTokenizer

bolot@cc.gatech.edu

breaks the stream of characters into a stream of XMLnodes (aka token stream)
token stream is used by XMLparser to generate XMLdocument tree!

XMLTokenizer subclass: #SAXDriver
	instanceVariableNames: 'saxHandler scoping useNamespaces validateAttributes languageEnvironment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Parser'!

!SAXDriver commentStamp: 'JAAyer 2/21/2010 17:57' prior: 0!
SAXDriver together with SAXHandler provides a SAX-like API for XML parsing. A SAXDriver processes XML tokens and signals SAX events to a SAXHandler. To use it, create a subclass of SAXHandler and override messages to handle SAX events of interest. See "instance creation" on the class-side of SAXHandler for ways to instantiate such a parser.!

!SAXDriver methodsFor: 'errors' stamp: 'JAAyer 3/2/2010 04:07'!
errorBadNamespace: aNamespace inEndTag: anElementName
	self parseError:
		('End tag ', anElementName,
		' refers to undefined namespace ', aNamespace asString)! !

!SAXDriver methodsFor: 'errors' stamp: 'JAAyer 3/2/2010 04:08'!
errorBadNamespace: aNamespace inStartTag: anElementName
	self parseError:
		('Start tag ', anElementName,
		' refers to undefined namespace ', aNamespace asString)! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'JAAyer 2/21/2010 15:13'!
handleCData: aString
	self saxHandler
		checkEOD;
		startCData;
		characters: aString;
		endCData! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'cwp 6/17/2003 18:26'!
handleComment: aString
	self saxHandler
		checkEOD; 
		comment: aString! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'mir 1/8/2002 18:38'!
handleEndDocument
	self saxHandler endDocument! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'JAAyer 3/3/2010 23:03'!
handleEndTag: elementName
	| namespace localName namespaceURI |

	self usesNamespaces
		ifTrue: [
			elementName splitQualifiedNameInto: [:ns :ln |
				namespace := ns.
				localName := ln].

			(namespace isNil or: [self scoping isBoundNamespace: namespace])
				ifFalse: [self errorBadNamespace: namespace inEndTag: elementName].

			"call the handler"
			self saxHandler
				checkEOD; 
				endElement: localName
				namespace: namespace
				qualifiedName: elementName
				scope: self scoping currentScope.
			self scoping leaveScope]
		ifFalse: [
			"call the handler"
			self saxHandler
				checkEOD; 
				endElement: elementName namespace: nil namespaceURI: nil qualifiedName: elementName]! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'mir 7/14/2006 11:55'!
handlePCData: aString
	self languageEnvironment
		ifNotNil: [aString applyLanguageInfomation: self languageEnvironment].
	self saxHandler
		checkEOD; 
		characters: aString! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'mir 1/8/2002 18:24'!
handlePI: piTarget data: piData
	self saxHandler
		checkEOD; 
		processingInstruction: piTarget data: piData! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'mir 8/14/2000 18:29'!
handleStartDocument
	self saxHandler startDocument! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'JAAyer 3/3/2010 23:05'!
handleStartTag: elementName attributes: attributeList namespaces: namespaces
	| localName namespace namespaceURI |

	(attributeList includesKey: 'xml:lang')
		ifTrue: [self languageEnvironment: (attributeList at: 'xml:lang')].
	self usesNamespaces
		ifTrue: [
			self scoping enterScope.
				"declare any namespaces"	
				namespaces keysAndValuesDo: [:ns :uri |
					self scoping declareNamespace: ns uri: uri].

			elementName splitQualifiedNameInto: [:ns :ln |
				namespace := ns.
				localName := ln].

			"ensure our namespace is defined"
			(namespace isNil or: [self scoping isBoundNamespace: namespace])
				ifFalse: [self errorBadNamespace: namespace inStartTag: elementName].
			self validatesAttributes
				ifTrue: [self scoping validateAttributes: attributeList].

			"call the handler"
			self saxHandler
				checkEOD; 
				startElement: localName
				namespace: namespace
				attributeList: attributeList
				scope: self scoping currentScope]
		ifFalse: [
			"call the handler"
			self saxHandler
				checkEOD; 
				startElement: elementName
				namespace: nil
				namespaceURI: nil
				attributeList: attributeList]! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'cwp 6/18/2003 01:00'!
handleWhitespace: aString
	self saxHandler
		checkEOD; 
		ignorableWhitespace: aString! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'JAAyer 3/2/2010 22:54'!
handleXMLDecl: attributes
	self saxHandler
		checkEOD; 
		documentAttributes: attributes! !

!SAXDriver methodsFor: 'initialization' stamp: 'mir 6/5/2003 16:29'!
initialize
	super initialize.
	useNamespaces := false.
	validateAttributes := false! !

!SAXDriver methodsFor: 'accessing' stamp: 'mir 7/14/2006 11:48'!
languageEnvironment
	^languageEnvironment! !

!SAXDriver methodsFor: 'accessing' stamp: 'JAAyer 3/2/2010 03:31'!
languageEnvironment: anIsoString
	languageEnvironment := LanguageEnvironment localeID: (LocaleID isoString: anIsoString)! !

!SAXDriver methodsFor: 'accessing' stamp: 'mir 8/11/2000 17:51'!
saxHandler
	^saxHandler! !

!SAXDriver methodsFor: 'accessing' stamp: 'mir 8/11/2000 17:52'!
saxHandler: aHandler
	saxHandler := aHandler! !

!SAXDriver methodsFor: 'namespaces' stamp: 'JAAyer 3/3/2010 22:26'!
scoping
	^ scoping ifNil: [scoping := XMLNamespaceScopeStack new]! !

!SAXDriver methodsFor: 'namespaces' stamp: 'mir 6/16/2003 13:09'!
splitName: aName into: twoArgsBlock
	"Split the name into namespace and local name (the block arguments).
	Handle both qualified and unqualified names using the default name space"

	| i ns ln |
	i := aName lastIndexOf: $:.
	i = 0
		ifTrue: [
			ns := nil.
			ln := aName]
		ifFalse: [
			ns := aName copyFrom: 1 to: (i - 1).
			ln := aName copyFrom: i+1 to: aName size].
	twoArgsBlock value: ns value: ln! !

!SAXDriver methodsFor: 'accessing' stamp: 'mir 6/24/2003 14:51'!
useNamespaces: aBoolean
	useNamespaces := aBoolean! !

!SAXDriver methodsFor: 'testing' stamp: 'mir 6/5/2003 16:30'!
usesNamespaces
	^useNamespaces! !

!SAXDriver methodsFor: 'testing' stamp: 'mir 6/5/2003 17:06'!
validatesAttributes
	^validateAttributes! !

XMLTokenizer subclass: #XMLParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Parser'!

!XMLParser commentStamp: 'JAAyer 2/21/2010 15:55' prior: 0!
This is a generic parser that supports rudimentary handling of elements and character data. For a more complete SAX-like API, see SAXHandler (and SAXDriver). For a DOM-like API, see XMLDOMParser and the classes under XML-Parser-Nodes.!

!XMLParser methodsFor: 'callbacks' stamp: 'SqR 7/2/2000 16:51'!
attribute: aSymbol value: aString
	"This method is called for each attribute/value pair in a start tag"

	^self subclassResponsibility! !

!XMLParser methodsFor: 'callbacks' stamp: 'SqR 7/2/2000 16:52'!
beginStartTag: aSymbol asPI: aBoolean
	"This method is called for at the beginning of a start tag.
	The asPI parameter defines whether or not the tag is a 'processing
	instruction' rather than a 'normal' tag."

	^self subclassResponsibility! !

!XMLParser methodsFor: 'callbacks' stamp: 'SqR 7/2/2000 16:52'!
endStartTag: aSymbol
	"This method is called at the end of the start tag after all of the
	attributes have been processed"

	^self subclassResponsibility! !

!XMLParser methodsFor: 'callbacks' stamp: 'SqR 7/2/2000 16:52'!
endTag: aSymbol
	"This method is called when the parser encounters either an
	end tag or the end of a unary tag"

	^self subclassResponsibility! !

!XMLParser methodsFor: 'handling tokens' stamp: 'mir 1/17/2002 09:27'!
handleCData: aString
	self text: aString! !

!XMLParser methodsFor: 'handling tokens' stamp: 'mir 1/17/2002 09:26'!
handleEndTag: aString
	self endTag: aString! !

!XMLParser methodsFor: 'handling tokens' stamp: 'mir 1/17/2002 09:27'!
handlePCData: aString
	self text: aString! !

!XMLParser methodsFor: 'handling tokens' stamp: 'mir 1/17/2002 09:26'!
handleStartTag: tagName attributes: attributes
	self beginStartTag: tagName asPI: false.
	attributes keysAndValuesDo: [:key :value |
		self attribute: key value: value].
	self endStartTag: tagName! !

!XMLParser methodsFor: 'callbacks' stamp: 'SqR 7/2/2000 16:52'!
text: aString
	"This method is called for the blocks of text between tags.
	It preserves whitespace, but has all of the enclosed entities expanded"

	^self subclassResponsibility! !

!XMLTokenizer class methodsFor: 'class initialization' stamp: 'mir 10/31/2007 17:22'!
initialize
	"XMLTokenizer initialize"

	CharEscapes := CharacterSet newFrom: #( $& $" $' $> $< ).

	SeparatorTable  := CharacterSet new.
	#(9 10 12 13 32) do: [:each | SeparatorTable add: each asCharacter].

	LiteralChars := CharacterSet newFrom: #( $: $- $: $= $.).
	0 to: 255 do: [:i | 
		| char |
		char := i asCharacter.
		(char isDigit or: [char isLetter])
		ifTrue: [LiteralChars add: char]].

	NameDelimiters := CharacterSet new.
	#(9 10 12 13 32 61 "$= asInteger 61" 62 "$> asInteger" 47 "$/ asInteger")
		do: [:each | NameDelimiters add: each asCharacter].

	DigitTable := Array new: 256.
	DigitTable atAllPut: -1.
	($0 to: $9) do: [:each | DigitTable at: each asciiValue put: each digitValue].
	($a to: $f) do: [:each | DigitTable at: each asciiValue put: each digitValue].
	($A to: $F) do: [:each | DigitTable at: each asciiValue put: each digitValue].
! !

!XMLTokenizer class methodsFor: 'accessing' stamp: 'mir 11/23/2007 15:50'!
isCharEscape: entityValue
	^entityValue size = 1
		and: [CharEscapes includes: entityValue first]! !

!XMLTokenizer class methodsFor: 'instance creation' stamp: 'mir 11/16/2000 07:58'!
on: aStream
	^self new parseStream: aStream! !

!XMLTokenizer methodsFor: 'streaming' stamp: 'mir 3/14/2003 22:58'!
atEnd
	nestedStreams == nil
		ifTrue: [^peekChar == nil and: [stream atEnd]].
	^stream atEnd
		ifTrue: [
			self popNestingLevel.
			self atEnd]
		ifFalse: [false]! !

!XMLTokenizer methodsFor: 'streaming' stamp: 'GeorgeHerolyants 1/9/2010 23:28'!
basicNext
	"Returns next character in the stream after performing line endings normalization"
	| char nextChar |
	
	char := stream next.
	char == Character cr 
		ifTrue: [
			char := Character lf.
			nextChar := stream next.
			(nextChar = Character lf or: [nextChar == nil]) ifFalse: [self pushBack: nextChar asString]].
	^ char! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'mir 1/17/2002 18:12'!
checkAndExpandReference: parsingContext
	| referenceString nextChar |
	nextChar := self peek.
	self validating
		ifFalse: [^nil].
	nextChar == $&
		ifTrue: [
			self next.
			self peek == $#
				ifTrue: [^self pushStream: (ReadStream on: self nextCharReference asString)].
			referenceString := self nextLiteral.
			self next == $;
				ifFalse: [self errorExpected: ';'].
			self handleEntity: referenceString in: parsingContext ]
		ifFalse: [
			((nextChar == $%
				and: [self parsingMarkup])
				and: [parsingContext == #entityValue])
				ifTrue: [
					self skipSeparators.
					referenceString := self nextLiteral.
					self handleEntity: referenceString in: parsingContext]].

	self atEnd ifTrue: [self errorExpected: 'Character expected.'].
	^nextChar! !

!XMLTokenizer methodsFor: 'streaming' stamp: 'mir 3/14/2003 22:59'!
checkNestedStream
	nestedStreams == nil
		ifFalse: [(peekChar == nil and: [self stream atEnd])
			ifTrue: [
				self popNestingLevel.
				self checkNestedStream]]
! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'mir 11/16/2000 21:41'!
conditionalInclude: conditionalKeyword
	conditionalKeyword = 'INCLUDE'
		ifTrue: [^true].
	conditionalKeyword = 'IGNORE'
		ifTrue: [^false].
	^self conditionalInclude: (self parameterEntity: conditionalKeyword) value! !

!XMLTokenizer methodsFor: 'tokenizing dtd' stamp: 'mir 6/29/2001 00:08'!
endDocTypeDecl
	"Skip ]>"
	self next; next.
	^nil! !

!XMLTokenizer methodsFor: 'private' stamp: 'mir 11/13/2000 18:19'!
endParsingMarkup
	parsingMarkup := false! !

!XMLTokenizer methodsFor: 'entities' stamp: 'mir 1/14/2002 15:06'!
entities
	entities ifNil: [entities := self initEntities].
	^entities! !

!XMLTokenizer methodsFor: 'entities' stamp: 'mir 1/17/2002 13:53'!
entity: refName
	^self validating
		ifTrue: [self entities
			at: refName
			ifAbsentPut: [self parseError: 'XML undefined entity ' , refName printString]]
		ifFalse: [DTDEntityDeclaration name: refName value: '']
! !

!XMLTokenizer methodsFor: 'entities' stamp: 'mir 11/16/2000 21:43'!
entity: refName put: aReference
	"Only the first declaration of an entity is valid so if there is already one don't register the new value."
	self entities at: refName ifAbsentPut: [aReference]! !

!XMLTokenizer methodsFor: 'errors' stamp: 'mir 5/14/2003 18:27'!
errorExpected: expectedString
	| actualString |
	actualString := ''.
	self atEnd
		ifFalse: [
			[actualString := self next: 20]
				on: Error
				do: [:ex | ]].
	self parseError: 'XML expected ' , expectedString printString , ': ' , actualString! !

!XMLTokenizer methodsFor: 'entities' stamp: 'mir 1/14/2002 17:59'!
externalEntities
	externalEntities ifNil: [externalEntities := Dictionary new].
	^externalEntities! !

!XMLTokenizer methodsFor: 'entities' stamp: 'mir 1/14/2002 17:59'!
externalEntity: refName
	^self entities
		at: refName
		ifAbsentPut: ['']! !

!XMLTokenizer methodsFor: 'private' stamp: 'mir 3/14/2003 23:09'!
fastStreamStringContents: writeStream
	| newSize |
	newSize := writeStream position.
	^(String new: newSize)
		replaceFrom: 1
		to: newSize
		with: writeStream originalContents
		startingAt: 1! !

!XMLTokenizer methodsFor: 'handling tokens' stamp: 'mir 11/13/2000 16:04'!
handleCData: aString
	self log: 'CData: ' , aString! !

!XMLTokenizer methodsFor: 'handling tokens' stamp: 'mir 8/14/2000 11:37'!
handleComment: aString
	self log: 'Comment: ' , aString! !

!XMLTokenizer methodsFor: 'handling tokens' stamp: 'mir 8/14/2000 18:27'!
handleEndDocument
	self log: 'End Doc '! !

!XMLTokenizer methodsFor: 'handling tokens' stamp: 'mir 8/14/2000 11:38'!
handleEndTag: aString
	self log: 'End tag: ' , aString! !

!XMLTokenizer methodsFor: 'entities' stamp: 'mir 1/17/2002 18:12'!
handleEntity: referenceString in: parsingContext 

	| entity entityValue |
	entity := self entity: referenceString.
	entityValue := entity valueForContext: parsingContext.
	(self class isCharEscape: entityValue)
		ifTrue: [entityValue := entity reference].
	self pushStream: (ReadStream on: entityValue asString)! !

!XMLTokenizer methodsFor: 'handling tokens' stamp: 'mir 8/14/2000 11:38'!
handlePCData: aString
	self log: 'PCData: ' , aString! !

!XMLTokenizer methodsFor: 'handling tokens' stamp: 'mir 12/11/2000 16:10'!
handlePI: piTarget data: piData
	self log: 'PI: ' , piTarget , ' data ' , piData! !

!XMLTokenizer methodsFor: 'handling tokens' stamp: 'mir 8/14/2000 18:26'!
handleStartDocument
	self log: 'Start Doc'! !

!XMLTokenizer methodsFor: 'handling tokens' stamp: 'mir 8/14/2000 11:39'!
handleStartTag: tagName attributes: attributes
	self log: 'Start tag: ' , tagName.
	attributes keysAndValuesDo: [:key :value |
		self log: key , '->' , value]! !

!XMLTokenizer methodsFor: 'handling tokens' stamp: 'cwp 6/17/2003 21:08'!
handleWhitespace: aString
	self log: 'Whitespace: ' , aString! !

!XMLTokenizer methodsFor: 'handling tokens' stamp: 'JAAyer 3/2/2010 22:54'!
handleXMLDecl: attributes
	attributes keysAndValuesDo: [:key :value |
		self log: key , '->' , value]! !

!XMLTokenizer methodsFor: 'streaming' stamp: 'mir 6/28/2001 16:45'!
hasNestedStreams
	^nestedStreams notNil! !

!XMLTokenizer methodsFor: 'entities' stamp: 'mir 11/23/2007 15:45'!
initEntities
	| ents |
	ents := Dictionary new.
	ents
		at: 'amp' put: (DTDEntityDeclaration name: 'amp' value: '&');
		at: 'quot' put: (DTDEntityDeclaration name: 'quot' value: '"');
		at: 'apos' put: (DTDEntityDeclaration name: 'apos' value: '''');
		at: 'gt' put: (DTDEntityDeclaration name: 'gt' value: '>');
		at: 'lt' put: (DTDEntityDeclaration name: 'lt' value: '<').
	^ents! !

!XMLTokenizer methodsFor: 'initialize' stamp: 'mir 3/14/2003 19:27'!
initialize
	parsingMarkup := false.
	validating := false.
	attributeBuffer := WriteStream on: (String new: 128).
	nameBuffer := WriteStream on: (String new: 128)! !

!XMLTokenizer methodsFor: 'private' stamp: 'mir 12/7/2000 16:46'!
log: aString
	"Transcript show: aString; cr"! !

!XMLTokenizer methodsFor: 'errors' stamp: 'mir 1/9/2002 15:26'!
malformedError: errorString
	SAXMalformedException signal: errorString! !

!XMLTokenizer methodsFor: 'streaming' stamp: 'mir 11/23/2007 18:16'!
match: subCollection into: resultStream
	"Set the access position of the receiver to be past the next occurrence of the subCollection. Answer whether subCollection is found.  No wildcards, and case does matter."

	| pattern startMatch |
	pattern _ ReadStream on: subCollection.
	startMatch _ nil.
	[pattern atEnd] whileFalse: 
		[self atEnd ifTrue: [^ false].
		(self next) = (pattern next) 
			ifTrue: [pattern position = 1 ifTrue: [startMatch _ self position]]
			ifFalse: [pattern position: 0.
					startMatch ifNotNil: [
						self position: startMatch.
						startMatch _ nil]]].
	^ true

! !

!XMLTokenizer methodsFor: 'private' stamp: 'mir 6/28/2001 16:54'!
nestedStreams
	nestedStreams ifNil: [nestedStreams := OrderedCollection new].
	^nestedStreams! !

!XMLTokenizer methodsFor: 'streaming' stamp: 'GeorgeHerolyants 1/9/2010 20:32'!
next
	"Return the next character from the current input stream. If the current stream is at end pop to next nesting level if there is one.
	Due to the potential nesting of original document, included documents and replacment texts the streams are held in a stack representing the nested streams. The current stream is the top one."
	| nextChar |
	peekChar
		ifNil: [
			nestedStreams ifNotNil: [self checkNestedStream].
			^nextChar := self basicNext]
		ifNotNil: [
			nextChar := peekChar.
			peekChar := nil.
			^nextChar].
	! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'mir 6/5/2003 16:32'!
nextAttributeInto: attributes namespaces: namespaces

	| attrName attrValue |
	attrName := self nextName.
	self skipSeparators.
	self next == $=
		ifFalse: [self errorExpected: '='].
	self skipSeparators.
	attrValue := self nextAttributeValue.

	(self usesNamespaces
		and: [(attrName findString: 'xmlns') = 1])
		ifTrue: [attrName size > 6
			ifTrue: [namespaces at: (attrName copyFrom: 7 to: attrName size) put: attrValue]
			ifFalse: [namespaces at: attrName put: attrValue]]
		ifFalse: [attributes at: attrName put: attrValue]! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'mir 11/23/2007 17:51'!
nextAttributeValue
	| delimiterChar attributeValueStream nextChar nextPeek referenceString entity entityValue |
	delimiterChar := self next.
	(delimiterChar == $"
		or: [delimiterChar == $'])
		ifFalse: [self errorExpected: 'Attribute value delimiter expected.'].
	attributeValueStream := attributeBuffer reset.
	[
	nextPeek := nextChar := self next.
	nextChar ifNil: [self errorExpected: 'Character expected.'].
	nextChar == $&
		ifTrue: [
			self peek == $#
				ifTrue: [
					nextPeek := nil.
					nextChar := self nextCharReference]
				ifFalse: [
					referenceString := self nextLiteral.
					self next == $;
						ifFalse: [self errorExpected: ';'].
					entity := self entity: referenceString.
					entityValue := entity valueForContext: #content.
					(self class isCharEscape: entityValue)
						ifTrue: [
							nextPeek := nil.
							nextChar := entityValue first]
						ifFalse: [
							entityValue := entityValue asString.
							entityValue isEmpty
								ifTrue: [nextPeek := nextChar := nil]
								ifFalse: [
									self pushStream: (ReadStream on: entityValue asString).
									nextPeek := nextChar := self next]]]].
	nextPeek == delimiterChar]
		whileFalse: [
			nextChar ifNotNil: [attributeValueStream nextPut: nextChar]].
	^self fastStreamStringContents: attributeValueStream
"	^attributeValueStream contents"! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'mir 1/17/2002 17:00'!
nextCDataContent
	| cdata |
	"Skip $[ "
	self next.
	cdata := self nextUpToAll: ']]>'.
	self handleCData: cdata
! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'mir 12/6/2000 14:29'!
nextCDataOrConditional

	| nextChar conditionalKeyword |
	"Skip ["
	self next.
	self skipSeparators.
	nextChar := self peek.
	nextChar == $%
		ifTrue: [
			self checkAndExpandReference: (self parsingMarkup ifTrue: [#dtd] ifFalse: [#content]).
			conditionalKeyword := self nextLiteral.
			self skipSeparators.
			^self next == $[
				ifTrue: [
						self skipSeparators.
						self nextIncludeSection: (self conditionalInclude: conditionalKeyword)]
				ifFalse: [self errorExpected: '[' ]].

	nextChar == $C
		ifTrue: [
			^self nextLiteral = 'CDATA'
				ifTrue: [self peek == $[
							ifTrue: [self nextCDataContent]
							ifFalse: [self errorExpected: '[' ]]
				ifFalse: [self errorExpected: 'CData']].
	self errorExpected: 'CData or declaration'
! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'mu 11/14/2005 14:07'!
nextCharReference
	| base charValue |
	self next == $#
		ifFalse: [self errorExpected: 'character reference'].
	base := self peek == $x
		ifTrue: [
			self next.
			16]
		ifFalse: [10].

	charValue := [self readNumberBase: base] on: Error do: [:ex | self errorExpected: 'Number.'].
	(self next) == $;
		ifFalse: [self errorExpected: '";"'].
	^Unicode value: charValue! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'mir 11/28/2000 17:54'!
nextComment
	| string |
	"Skip first -"
	self next.
	self next == $-
		ifFalse: [self errorExpected: 'second comment $-'].
	string := self nextUpToAll: '-->'.
	self handleComment: string! !

!XMLTokenizer methodsFor: 'tokenizing dtd' stamp: 'mir 1/8/2002 13:54'!
nextDocType
	| declType |
	declType := self nextLiteral.
	declType = 'DOCTYPE'
		ifTrue: [
			self startParsingMarkup.
			^self nextDocTypeDecl].
	self errorExpected: 'markup declaration, not ' , declType printString! !

!XMLTokenizer methodsFor: 'tokenizing dtd' stamp: 'mir 1/17/2002 17:29'!
nextDocTypeDecl
	| nextChar |
	self skipSeparators.
	self nextLiteral.
	self skipSeparators.
	self peek == $[
		ifFalse: [[nextChar := self peek.
				nextChar == $> or: [nextChar == $[ ]] whileFalse: [self next]].
	self peek == $[
		ifTrue: [
			self next.
			[self skipSeparators.
			self peek == $]] whileFalse: [
				self checkAndExpandReference: #dtd.
				self nextNode].
			self next == $] 
				ifFalse: [self errorExpected: ']' ]].
	self skipSeparators.
	self next == $>
		ifFalse: [self errorExpected: '>' ].

	self endParsingMarkup! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'mir 11/23/2007 17:48'!
nextEndTag
	| tagName |
	"Skip /"
	self next.
	tagName := self nextName.
	self skipSeparators.
	(self nextTrimmedBlanksUpTo: $>)
		ifNotEmpty: [self parseError: 'XML invalid end tag ' , tagName].
	self handleEndTag: tagName! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'gvc 6/6/2007 13:17'!
nextEntity
	"return the next XMLnode, or nil if there are no more.
	Fixed to retain leading whitespace when PCDATA is detected."

	|whitespace|
	"branch, depending on what the first character is"
	whitespace := self nextWhitespace.
	self atEnd ifTrue: [self handleEndDocument. ^ nil].
	self checkAndExpandReference: (self parsingMarkup ifTrue: [#dtd] ifFalse: [#content]).
	^self peek = $<
		ifTrue: [self nextNode]
		ifFalse: [whitespace isEmpty
					ifFalse: [self pushBack: whitespace].
				self nextPCData]! !

!XMLTokenizer methodsFor: 'tokenizing dtd' stamp: 'mir 1/17/2002 14:24'!
nextEntityDeclaration
	| entityName entityDef referenceClass reference |
	self skipSeparators.
	referenceClass := self peek == $%
		ifTrue: [
			self next.
			self skipSeparators.
			DTDParameterEntityDeclaration]
		ifFalse: [DTDEntityDeclaration].
	entityName := self nextLiteral.
	self skipSeparators.
	entityDef := (self peek == $" or: [self peek == $'])
		ifTrue: [self nextEntityValue]
		ifFalse: [self nextExternalId].
	self skipUpTo: $>.
	reference := referenceClass name: entityName value: entityDef.
	reference registerIn: self.
	^reference! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'mir 1/17/2002 18:14'!
nextEntityValue
	| delimiterChar entityValueStream nextChar nextPeek referenceString entity entityValue |
	delimiterChar := self next.
	(delimiterChar == $"
		or: [delimiterChar == $'])
		ifFalse: [self errorExpected: 'Entity value delimiter expected.'].

	entityValueStream := WriteStream on: (String new).
	[
	nextPeek := nextChar := self peek.
	nextChar ifNil: [self errorExpected: 'Character expected.'].
	nextChar == $&
		ifTrue: [
			self next.
			self peek == $#
				ifTrue: [
					nextPeek := nil.
					nextChar := self nextCharReference]
				ifFalse: [
					referenceString := self nextLiteral.
					self next == $;
						ifFalse: [self errorExpected: ';'].
					entity := self entity: referenceString.
					entityValue := entity valueForContext: #entityValue.
					self pushStream: (ReadStream on: entityValue asString).
					nextPeek := nextChar := self next]]
		ifFalse: [
			nextChar == $%
				ifTrue: [
					self skipSeparators.
					referenceString := self nextLiteral.
					nextChar := self handleEntity: referenceString in: #entityValue.
					nextPeek := nextChar := self next]
				ifFalse: [self next]].
	nextPeek == delimiterChar]
		whileFalse: [
			nextChar ifNotNil: [entityValueStream nextPut: nextChar]].
	^entityValueStream contents! !

!XMLTokenizer methodsFor: 'tokenizing dtd' stamp: 'rkris 7/28/2004 12:35'!
nextExternalId
	| extDefType systemId dir |
	extDefType := self nextLiteral.
	extDefType = 'PUBLIC'
		ifTrue: [
			self skipSeparators.
			self nextPubidLiteral.
			self skipSeparators.
			self peek == $>
				ifFalse: [
					systemId := self nextSystemLiteral]].

	extDefType = 'SYSTEM'
		ifTrue: [
			self skipSeparators.
			systemId := self nextSystemLiteral].

	systemId
		ifNil: [^nil].

	"The rest of this method only applies if we're reading aFileStream"
	(self topStream isKindOf: FileStream)
		ifFalse: [^''].
	dir := self topStream directory.
	^(dir fileExists: systemId)
		ifTrue: [(dir readOnlyFileNamed: systemId) contentsOfEntireFile]
		ifFalse: ['']! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'mir 6/28/2001 16:38'!
nextIncludeSection: parseSection
	| section |
	"Read the file up to the next include section delimiter and parse it if parseSection is true"

	
	section := self nextUpToAll: ']]>'.
	parseSection
		ifTrue: [
			self pushStream: (ReadStream on: section)]! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'mir 10/31/2007 16:19'!
nextLiteral
	| resultStream nextChar resultString |
	resultStream := (String new: 10) writeStream.
	((nextChar := self peek) isLetter
		or: [nextChar == $_])
		ifFalse: [self errorExpected: 'Name literal.'].
	[nextChar := self peek.
	(LiteralChars includes: nextChar)
		ifTrue: [
			nextChar == $&
				ifTrue: [
					nextChar := self next.
					resultStream nextPut: (self peek == $#
						ifTrue: [self nextCharReference]
						ifFalse: [^resultStream contents])]
				ifFalse: [
					resultStream nextPut: self next]]
		ifFalse: [resultString := resultStream contents.
			resultString isEmpty
				ifTrue: [self errorExpected: 'Name literal']
				ifFalse: [^resultString]]] repeat! !

!XMLTokenizer methodsFor: 'tokenizing dtd' stamp: 'mir 1/17/2002 13:49'!
nextMarkupDeclaration
	| declType |
	declType := self nextLiteral.
	self validating
		ifFalse: [^self skipMarkupDeclaration].
	declType = 'ENTITY'
		ifTrue: [self nextEntityDeclaration]
		ifFalse: [self skipMarkupDeclaration]! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'Alexandre Bergel 3/4/2010 15:04'!
nextName
	| nextChar |
	nameBuffer reset.
	self peek == $.
		ifTrue: [self malformedError: 'Character expected.'].
	[(nextChar := self peek)
		ifNil: [self errorExpected: 'Character expected.'].
	NameDelimiters includes: nextChar] whileFalse: [
			nameBuffer nextPut: self next].
	^ (self fastStreamStringContents: nameBuffer) ! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'mir 11/28/2000 17:52'!
nextNode
	| nextChar |
	"Skip < "
	self next.
	nextChar := self peek.
	nextChar == $!! ifTrue: [
		"Skip !!"
		self next.
		nextChar := self peek.
		nextChar == $- ifTrue: [^self nextComment].
		nextChar == $[ ifTrue: [^self nextCDataOrConditional].
		^self parsingMarkup
			ifTrue: [self nextMarkupDeclaration]
			ifFalse: [self nextDocType]].
	nextChar == $? ifTrue: [^self nextPI].
	^self nextTag! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'mir 11/23/2007 17:52'!
nextPCData
	| resultStream nextChar referenceString entity entityValue nextPeek |
	resultStream := (String new: 10) writeStream.
	self validating
		ifFalse: [
			[self peek == $<]
				whileFalse: [resultStream nextPut: self next].
			^self handlePCData: resultStream contents].

	[
	nextPeek := nextChar := self peek.
	nextChar ifNil: [self errorExpected: 'Character expected.'].
	nextChar == $&
		ifTrue: [
			self next.
			self peek == $#
				ifTrue: [
					nextPeek := nil.
					nextChar := self nextCharReference]
				ifFalse: [
					referenceString := self nextLiteral.
					self next == $;
						ifFalse: [self errorExpected: ';'].
					entity := self entity: referenceString.
					entityValue := entity valueForContext: #content.
					(self class isCharEscape: entityValue)
						ifTrue: [
							nextPeek := nil.
							nextChar := entityValue first]
						ifFalse: [
							entityValue := entityValue asString.
							entityValue isEmpty
								ifTrue: [nextPeek := nextChar := nil]
								ifFalse: [
									self pushStream: (ReadStream on: entityValue asString).
									nextPeek := nextChar := self peek]]]]
		ifFalse: [nextPeek == $< ifFalse: [self next]].
	nextPeek == $<]
		whileFalse: [
			nextChar ifNotNil: [resultStream nextPut: nextChar]].
	self handlePCData: resultStream contents! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'mir 1/17/2002 13:00'!
nextPI
	| piTarget piData |
	"Skip ?"
	self next.
	piTarget := self nextLiteral.
	piTarget asUppercase = 'XML'
		ifTrue: [^self nextXMLDecl].
	self skipSeparators.
	piData := self nextUpToAll: '?>'.
	self handlePI: piTarget data: piData! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'mir 1/17/2002 14:25'!
nextPubidLiteral
	^self nextAttributeValue! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'mir 1/17/2002 14:25'!
nextSystemLiteral
	^self nextAttributeValue! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'mir 6/5/2003 22:37'!
nextTag
	| tagName attributes nextChar namespaces |
	(self peek = $/)
		ifTrue: [^self nextEndTag].
	tagName := self nextName.
	self skipSeparators.
	attributes := Dictionary new: 33.
	namespaces := Dictionary new: 5.
	[(nextChar := self peek) == $> or: [nextChar == $/]] whileFalse: [
		self checkAndExpandReference: #content.
		self nextAttributeInto: attributes namespaces: namespaces.
		self skipSeparators.].
	self handleStartTag: tagName attributes: attributes namespaces: namespaces.
	self next == $/
		ifTrue: [
			self handleEndTag: tagName.
			self next].
	! !

!XMLTokenizer methodsFor: 'streaming' stamp: 'bf 4/27/2006 17:55'!
nextTrimmedBlanksUpTo: delimiter
	| resultStream nextChar |
	resultStream := WriteStream on: (String new: 10).
	nextChar := nil.
	[(nextChar := self next) == delimiter]
		whileFalse: [
			nextChar == $  ifFalse: [
				resultStream nextPut: nextChar]].
	nextChar == delimiter
		ifFalse: [self parseError: 'XML no delimiting ' , delimiter printString , ' found'].
	^resultStream contents
! !

!XMLTokenizer methodsFor: 'streaming' stamp: 'mir 5/14/2003 18:44'!
nextUpTo: delimiter
	| resultStream nextChar |
	resultStream := WriteStream on: (String new: 10).
	[self atEnd or: [(nextChar := self next) == delimiter]]
		whileFalse: [resultStream nextPut: nextChar].
	nextChar == delimiter
		ifFalse: [self parseError: 'XML no delimiting ' , delimiter printString , ' found'].
	^resultStream contents
! !

!XMLTokenizer methodsFor: 'streaming' stamp: 'mir 11/23/2007 19:42'!
nextUpToAll: delimitingString
	| string |
	self unpeek.
	string := self upToAll: delimitingString.
	string
		ifNil: [self parseError: 'XML no delimiting ' , delimitingString printString , ' found'].
	^string! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'mir 10/31/2007 17:49'!
nextWhitespace
	| nextChar resultStream resultString|
	resultStream := (String new: 10) writeStream.
	[((nextChar := self peek) ~~ nil)
		and: [SeparatorTable includes: nextChar]]
		whileTrue: [resultStream nextPut: nextChar. self next].
	(nestedStreams == nil or: [self atEnd not])
		ifFalse: [self checkNestedStream.
				self nextWhitespace].
	resultString := resultStream contents.
	resultString isEmpty ifFalse: [self handleWhitespace: resultString].
	^resultString! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 3/2/2010 22:56'!
nextXMLDecl
	| attributes nextChar namespaces |
	self skipSeparators.
	attributes := Dictionary new.
	namespaces := Dictionary new.
	[(nextChar := self peek) == $?] whileFalse: [
		self nextAttributeInto: attributes namespaces: namespaces.
		self skipSeparators.].
	self next.
	self next == $>
		ifFalse: [self errorExpected: '> expected.'].
	(attributes includesKey: 'encoding') ifTrue: [self streamEncoding: (attributes at: 'encoding')].
	self handleXMLDecl: attributes.! !

!XMLTokenizer methodsFor: 'entities' stamp: 'mir 11/16/2000 21:20'!
parameterEntities
	parameterEntities ifNil: [parameterEntities := Dictionary new].
	^parameterEntities! !

!XMLTokenizer methodsFor: 'entities' stamp: 'mir 11/16/2000 21:40'!
parameterEntity: refName
	^self parameterEntities
		at: refName
		ifAbsent: [self parseError: 'XML undefined parameter entity ' , refName printString]! !

!XMLTokenizer methodsFor: 'entities' stamp: 'mir 11/16/2000 21:42'!
parameterEntity: refName put: aReference
	"Only the first declaration of an entity is valid so if there is already one don't register the new value."
	self parameterEntities at: refName ifAbsentPut: [aReference]! !

!XMLTokenizer methodsFor: 'errors' stamp: 'mir 1/8/2002 15:37'!
parseError: errorString
	SAXParseException signal: errorString! !

!XMLTokenizer methodsFor: 'accessing' stamp: 'mir 6/28/2001 16:51'!
parseStream: aStream
	self stream: aStream! !

!XMLTokenizer methodsFor: 'private' stamp: 'mir 11/13/2000 18:19'!
parsingMarkup
	^parsingMarkup! !

!XMLTokenizer methodsFor: 'streaming' stamp: 'GeorgeHerolyants 1/9/2010 20:32'!
peek
	"Return the next character from the current input stream. If the current stream poop to next nesting level if there is one.
	Due to the potential nesting of original document, included documents and replacment texts the streams are held in a stack representing the nested streams. The current stream is the top one."
	peekChar
		ifNil: [
			nestedStreams ifNotNil: [self checkNestedStream].
			^peekChar := self basicNext]
		ifNotNil: [^peekChar]! !

!XMLTokenizer methodsFor: 'streaming' stamp: 'mir 6/29/2001 00:36'!
popNestingLevel
	self hasNestedStreams
		ifTrue: [
			self stream close.
			self stream: self nestedStreams removeLast.
			self nestedStreams size > 0
				ifFalse: [nestedStreams := nil]]! !

!XMLTokenizer methodsFor: 'streaming' stamp: 'gvc 6/6/2007 13:13'!
pushBack: aString
	"Fixed to push the string before the peek char (if any)."
	
	| pushBackString |
	pushBackString := peekChar
		ifNil: [aString]
		ifNotNil: [aString, peekChar asString].
	peekChar := nil.
	self pushStream: (ReadStream on: pushBackString)! !

!XMLTokenizer methodsFor: 'streaming' stamp: 'mir 1/16/2002 10:54'!
pushStream: newStream
	"Continue parsing from the new nested stream."
	self unpeek.
	self nestedStreams addLast: self stream.
	self stream: newStream! !

!XMLTokenizer methodsFor: 'private' stamp: 'mu 11/14/2005 14:12'!
readNumberBase: base
	"Read a hex number from stream until encountering $; "

	| value digit |

	base = 10 ifFalse: [	| numberString | 
		numberString := self nextUpTo: $;.
		self stream skip: -1.
		^Integer readFrom: numberString asUppercase readStream base: base. 
	].

	value := 0.
	digit := DigitTable at: self peek asciiValue.
	digit < 0
		ifTrue: [self error: 'At least one digit expected here'].
	self next.
	value := digit.
	[digit := DigitTable at: self peek asciiValue.
	digit < 0
		ifTrue: [^value]
		ifFalse: [
			self next.
			value := value * base + digit]
		] repeat.
	^ value! !

!XMLTokenizer methodsFor: 'tokenizing dtd' stamp: 'mir 1/4/2002 11:05'!
skipMarkupDeclaration
	self skipUpTo: $>! !

!XMLTokenizer methodsFor: 'streaming' stamp: 'mir 10/31/2007 16:33'!
skipSeparators
	| nextChar |
	[((nextChar := self peek) ~~ nil)
		and: [SeparatorTable includes: nextChar]]
		whileTrue: [self next].
	(nestedStreams == nil or: [self atEnd not])
		ifFalse: [
			self checkNestedStream.
			self skipSeparators]! !

!XMLTokenizer methodsFor: 'streaming' stamp: 'mir 1/16/2002 10:42'!
skipUpTo: delimiter
	| nextChar |
	self unpeek.
	[self atEnd or: [(nextChar := self next) == delimiter]]
		whileFalse: [].
	nextChar == delimiter
		ifFalse: [self parseError: 'XML no delimiting ' , delimiter printString , ' found']
! !

!XMLTokenizer methodsFor: 'private' stamp: 'mir 11/13/2000 18:19'!
startParsingMarkup
	parsingMarkup := true! !

!XMLTokenizer methodsFor: 'private' stamp: 'mir 6/28/2001 16:50'!
stream
	^stream! !

!XMLTokenizer methodsFor: 'private' stamp: 'mir 6/28/2001 16:50'!
stream: newStream
	"Continue parsing from the new nested stream."
	stream := newStream! !

!XMLTokenizer methodsFor: 'streaming' stamp: 'mir 11/23/2007 19:40'!
stream: aStream upToAll: aCollection
	"Answer a subcollection from the current access position to the occurrence (not inclusive) of aCollection. If aCollection is not in the stream, answer nil."

	| startPos endMatch result |
	startPos := aStream position.
	(aStream  match: aCollection) 
		ifTrue: [endMatch := aStream position.
			aStream position: startPos.
			result := aStream next: endMatch - startPos - aCollection size.
			aStream position: endMatch.
			^ result]
		ifFalse: [
			aStream position: startPos.
			^nil]! !

!XMLTokenizer methodsFor: 'streaming' stamp: 'mir 7/12/2006 17:46'!
streamEncoding: encodingString

	| converterClass |
	Smalltalk at: #TextConverter ifPresent: [:tc | 
		(stream respondsTo: #converter:) ifTrue: [
			converterClass := tc defaultConverterClassForEncoding: encodingString asLowercase.
			converterClass ifNotNil: [stream converter: converterClass new]]]! !

!XMLTokenizer methodsFor: 'streaming' stamp: 'mir 1/17/2002 14:31'!
topStream
	^self hasNestedStreams
		ifTrue: [self nestedStreams first]
		ifFalse: [self stream]! !

!XMLTokenizer methodsFor: 'streaming' stamp: 'gvc 6/6/2007 13:19'!
unpeek
	"Fixed to use nested stream since multi-byte streams
	do not properly override pushBack: to deal with multi-byte
	characters."
	
	peekChar ifNotNil: [self pushBack: '']! !

!XMLTokenizer methodsFor: 'streaming' stamp: 'mir 11/23/2007 20:00'!
upToAll: delimitingString
	"Answer a subcollection from the current access position to the occurrence (if any, but not inclusive) of delimitingString. If delimitingString is not in the stream, answer the entire rest of the stream."

	| result |

	self hasNestedStreams
		ifFalse: [
			result := self stream: self stream upToAll: delimitingString.
			result
				ifNil: [self parseError: 'XML no delimiting ' , delimitingString printString , ' found'].
			^result].

	result := self stream: self stream upToAll: delimitingString.
	result
		ifNotNil: [^result].
	result := String streamContents: [:resultStream |
		resultStream nextPutAll: self stream upToEnd.
		self atEnd
			ifTrue: [self parseError: 'XML no delimiting ' , delimitingString printString , ' found'].
		self stream position timesRepeat: [
			self atEnd
				ifFalse: [
					resultStream nextPut: self next]]].
	self pushBack: result.
	^self upToAll: delimitingString! !

!XMLTokenizer methodsFor: 'testing' stamp: 'mir 6/5/2003 16:33'!
usesNamespaces
	^false! !

!XMLTokenizer methodsFor: 'testing' stamp: 'mir 1/14/2002 17:51'!
validating
	^validating! !

!XMLTokenizer methodsFor: 'accessing' stamp: 'mir 1/14/2002 17:51'!
validating: aBoolean
	validating := aBoolean! !

Object subclass: #XMLWriter
	instanceVariableNames: 'stream stack scoping canonical currentIndent indentString newLinesAllowed'
	classVariableNames: 'XMLTranslation XMLTranslationMap'
	poolDictionaries: ''
	category: 'XML-Parser-Parser'!

!XMLWriter classSide methodsFor: 'class initialization' stamp: 'ar 12/13/2002 01:04'!
initialize
	"XMLWriter initialize"

	XMLTranslation := Dictionary new.
	XMLTranslation
		at: Character cr put: '&#13;';
		at: Character lf put: '&#10;';
		at: Character tab put: '&#9;';
		at: $& put: '&amp;';
		at: $< put: '&lt;';
		at: $> put: '&gt;';
"		at: $' put: '&apos;'; "
		at: $" put: '&quot;'.
	XMLTranslationMap := ByteArray new: 256.
	XMLTranslation keysDo:[:ch| XMLTranslationMap at: ch asciiValue+1 put: 1].
! !

!XMLWriter classSide methodsFor: 'instance creation' stamp: 'mir 12/8/2000 17:54'!
on: aStream
	^self basicNew initialize stream: aStream! !

!XMLWriter methodsFor: 'writing xml' stamp: 'mir 5/20/2003 11:04'!
attribute: attributeName value: attributeValue
	self stream
		space;
		nextPutAll: attributeName.
	self
		eq;
		putAsXMLString: attributeValue! !

!XMLWriter methodsFor: 'accessing' stamp: 'mir 12/7/2000 15:54'!
canonical
	^canonical! !

!XMLWriter methodsFor: 'accessing' stamp: 'mir 12/7/2000 15:54'!
canonical: aBoolean
	canonical := aBoolean! !

!XMLWriter methodsFor: 'writing xml' stamp: 'mir 12/8/2000 17:55'!
cdata: aString
	self startCData.
	self stream nextPutAll: aString.
	self endCData! !

!XMLWriter methodsFor: 'writing xml' stamp: 'mir 12/8/2000 17:56'!
comment: aString
	self startComment.
	self stream nextPutAll: aString.
	self endComment! !

!XMLWriter methodsFor: 'private' stamp: 'GeorgeHerolyants 1/10/2010 18:53'!
cr
	self newLinesAllowed ifTrue: [self stream cr].! !

!XMLWriter methodsFor: 'namespaces' stamp: 'JAAyer 3/3/2010 07:36'!
declareNamespace: ns uri: uri
	self scoping declareNamespace: ns uri: uri! !

!XMLWriter methodsFor: 'private tags' stamp: 'GeorgeHerolyants 1/8/2010 19:50'!
endCData
	self stream nextPutAll: ']]>'.
	self cr.! !

!XMLWriter methodsFor: 'private tags' stamp: 'GeorgeHerolyants 1/8/2010 19:50'!
endComment
	self stream nextPutAll: ' -->'.
	self cr.! !

!XMLWriter methodsFor: 'writing dtd' stamp: 'mir 8/8/2000 18:13'!
endDecl: type
	self endTag! !

!XMLWriter methodsFor: 'writing dtd' stamp: 'mir 12/8/2000 18:02'!
endDeclaration
	self stream
		cr;
		nextPut: $].
	self endTag! !

!XMLWriter methodsFor: 'writing xml' stamp: 'JAAyer 3/3/2010 22:46'!
endElement: anElementName
	self scoping leaveScope.
	self endTag: anElementName.! !

!XMLWriter methodsFor: 'writing xml' stamp: 'GeorgeHerolyants 1/8/2010 22:26'!
endEmptyTag: tagName
	self popTag: tagName.
	self stream nextPutAll: '/>'.
	self canonical
		ifFalse: [self stream space].
	self cr.! !

!XMLWriter methodsFor: 'private tags' stamp: 'GeorgeHerolyants 1/8/2010 19:50'!
endPI
	self stream nextPutAll: ' ?>'.
	self cr.! !

!XMLWriter methodsFor: 'writing xml' stamp: 'GeorgeHerolyants 1/8/2010 19:48'!
endTag
	self stream nextPutAll: '>'.
	self cr.
	self indent.
	"self canonical
		ifFalse: [self stream space]"! !

!XMLWriter methodsFor: 'writing xml' stamp: 'JAAyer 3/3/2010 22:44'!
endTag: tagName
	self outdent.
	self popTag: tagName.
	self stream
		nextPutAll: '</';
		nextPutAll: tagName.
	self endTag.
! !

!XMLWriter methodsFor: 'namespaces' stamp: 'JAAyer 3/3/2010 08:00'!
enterScope
	self scoping enterScope! !

!XMLWriter methodsFor: 'private' stamp: 'mir 12/8/2000 18:00'!
eq
	self stream nextPut: $=! !

!XMLWriter methodsFor: 'writing xml' stamp: 'mir 5/20/2003 11:13'!
flush
	self stream flush! !

!XMLWriter methodsFor: 'private' stamp: 'mir 7/12/2006 16:52'!
indent
	currentIndent
		ifNotNil: [currentIndent := currentIndent +1]! !

!XMLWriter methodsFor: 'accessing' stamp: 'mir 7/12/2006 16:08'!
indentString: aString
	currentIndent := 0.
	indentString := aString! !

!XMLWriter methodsFor: 'accessing' stamp: 'mir 7/12/2006 16:08'!
indentTab
	self indentString: (String with: Character tab)! !

!XMLWriter methodsFor: 'initialize' stamp: 'JAAyer 3/3/2010 22:26'!
initialize
	stack := OrderedCollection new.
	canonical := false.
	scoping := XMLNamespaceScopeStack new.
	newLinesAllowed := false! !

!XMLWriter methodsFor: 'namespaces' stamp: 'JAAyer 3/3/2010 08:00'!
leaveScope
	self scoping leaveScope! !

!XMLWriter methodsFor: 'accessing' stamp: 'GeorgeHerolyants 1/10/2010 18:52'!
newLinesAllowed
	^ newLinesAllowed! !

!XMLWriter methodsFor: 'accessing' stamp: 'GeorgeHerolyants 1/10/2010 18:51'!
newLinesAllowed: aBoolean
	newLinesAllowed := aBoolean! !

!XMLWriter methodsFor: 'private' stamp: 'GeorgeHerolyants 1/10/2010 18:53'!
outdent
	currentIndent
		ifNotNil: [
			currentIndent := currentIndent-1.
			self writeIndent.
			currentIndent := currentIndent-1.]! !

!XMLWriter methodsFor: 'writing xml' stamp: 'GeorgeHerolyants 1/8/2010 20:08'!
pcData: aString
	self writeIndent.
	self writeXmlEncoded: aString.
	self cr.! !

!XMLWriter methodsFor: 'writing xml' stamp: 'mir 12/11/2000 16:12'!
pi: piTarget data: piData
	self startPI: piTarget.
	self stream nextPutAll: piData.
	self endPI! !

!XMLWriter methodsFor: 'private' stamp: 'mir 8/7/2000 16:23'!
popTag: tagName
	| stackTop |
	stackTop := self stack isEmpty
		ifTrue: ['<empty>']
		ifFalse: [self stack last].
	^stackTop = tagName
		ifTrue: [self stack removeLast]
		ifFalse: [self error: 'Closing tag "' , tagName , '" does not match "' , stackTop]! !

!XMLWriter methodsFor: 'accessing' stamp: 'GeorgeHerolyants 1/10/2010 19:20'!
prettyPrint
	self indentString: '    '.
	self newLinesAllowed: true.! !

!XMLWriter methodsFor: 'private' stamp: 'mir 8/7/2000 16:18'!
pushTag: tagName
	self stack add: tagName! !

!XMLWriter methodsFor: 'private' stamp: 'GeorgeHerolyants 1/8/2010 20:08'!
putAsXMLString: aValue
	self stream nextPut: $".
	self writeXmlEncoded: aValue.
	self stream nextPut: $"! !

!XMLWriter methodsFor: 'private' stamp: 'JAAyer 3/3/2010 07:35'!
scoping
	^ scoping! !

!XMLWriter methodsFor: 'private' stamp: 'mir 8/8/2000 17:02'!
stack
	^stack! !

!XMLWriter methodsFor: 'private tags' stamp: 'mir 12/8/2000 18:01'!
startCData
	self stream nextPutAll: '<!![CDATA['! !

!XMLWriter methodsFor: 'private tags' stamp: 'mir 12/8/2000 18:01'!
startComment
	self stream nextPutAll: '<-- '! !

!XMLWriter methodsFor: 'writing dtd' stamp: 'mir 12/8/2000 18:02'!
startDecl: type
	self stream
		nextPutAll: '<!!';
		nextPutAll: type asUppercase;
		space! !

!XMLWriter methodsFor: 'writing dtd' stamp: 'mir 12/8/2000 18:02'!
startDecl: type named: aString
	self stream
		nextPutAll: '<!!';
		nextPutAll: type asUppercase;
		space;
		nextPutAll: aString;
		space! !

!XMLWriter methodsFor: 'writing dtd' stamp: 'mir 12/8/2000 18:02'!
startDeclaration: dtdName
	self startDecl: 'DOCTYPE' named: dtdName.
	self stream
		nextPut: $[;
		cr! !

!XMLWriter methodsFor: 'writing xml' stamp: 'GeorgeHerolyants 1/8/2010 20:26'!
startElement: elementName attributeList: attributeList
	"self canonical
		ifFalse: [self stream cr]."
	self startTag: elementName.
	attributeList keys asSortedCollection do: [:key |
		self attribute: key value: (attributeList at: key)]! !

!XMLWriter methodsFor: 'writing xml' stamp: 'JAAyer 3/3/2010 22:36'!
startElement: elementName attributeList: attributeList scope: namespaceScope
	self startTag: elementName.
	self scoping enterScope.

	namespaceScope bindingsDo: [:namespace :uri |
		(self scoping isNamespace: namespace boundTo: uri)
			ifFalse: [self xmlns: namespace uri: uri]].

	(namespaceScope defaultNamespace ~= self scoping defaultNamespace)
		ifTrue: [self xmlns: nil uri: namespaceScope defaultNamespace].

	attributeList keys asSortedCollection do: [:key |
		self attribute: key value: (attributeList at: key)]! !

!XMLWriter methodsFor: 'private tags' stamp: 'mir 12/8/2000 18:01'!
startPI: identifier
	self stream
		nextPutAll: '<?';
		nextPutAll: identifier;
		space! !

!XMLWriter methodsFor: 'writing xml' stamp: 'JAAyer 3/3/2010 22:42'!
startTag: tagName
	self writeIndent.
	self stream
		nextPut: $<;
		nextPutAll: tagName.
	"self canonical
		ifFalse: [self stream space]."
	self pushTag: tagName! !

!XMLWriter methodsFor: 'accessing' stamp: 'mir 12/8/2000 17:54'!
stream
	^stream! !

!XMLWriter methodsFor: 'accessing' stamp: 'mir 12/8/2000 17:54'!
stream: aStream
	stream := aStream! !

!XMLWriter methodsFor: 'private' stamp: 'mir 7/12/2006 16:58'!
writeIndent
	currentIndent ifNotNil: [
		currentIndent timesRepeat: [self stream nextPutAll: indentString]]! !

!XMLWriter methodsFor: 'private' stamp: 'GeorgeHerolyants 1/8/2010 20:07'!
writeXmlEncoded: aString
	| lastIndex nextIndex |
	lastIndex := 1.
	"Unroll the first search to avoid copying"
	nextIndex := String findFirstInString: aString inSet: XMLTranslationMap startingAt: lastIndex.
	nextIndex = 0 ifTrue:[self stream nextPutAll: aString. ^ self].
	[self stream nextPutAll: (aString copyFrom: lastIndex to: nextIndex-1).
	self stream nextPutAll: (XMLTranslation at: (aString at: nextIndex)).
	lastIndex := nextIndex + 1.
	nextIndex := String findFirstInString: aString inSet: XMLTranslationMap startingAt: lastIndex.
	nextIndex = 0] whileFalse.
	self stream nextPutAll: (aString copyFrom: lastIndex to: aString size).! !

!XMLWriter methodsFor: 'writing xml' stamp: 'thf 9/2/2005 14:08'!
xmlDeclaration: versionString encoding: encodingString
	self canonical
		ifFalse: [
			self
				startPI: 'xml';
				attribute: 'version' value: versionString;
				attribute: 'encoding' value: encodingString;
				endPI.
			self stream flush]! !

!XMLWriter methodsFor: 'writing xml' stamp: 'JAAyer 3/3/2010 07:52'!
xmlns: namespace uri: uri
	self declareNamespace: namespace uri: uri.

	self
		attribute: (namespace ifNil: ['xmlns'] ifNotNil: ['xmlns:', namespace])
		value: uri! !

Error subclass: #SAXException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exception'!

SAXException subclass: #SAXMalformedException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exception'!

SAXException subclass: #SAXParseException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exception'!

Error subclass: #XMLException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exception'!

XMLException subclass: #XMLInvalidException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exception'!

XMLException subclass: #XMLMalformedException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exception'!

XMLException subclass: #XMLWarningException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exception'!

!String methodsFor: '*xml-parser' stamp: 'mir 7/14/2006 11:54'!
applyLanguageInfomation: languageEnvironment
	
	| leadingChar |
	leadingChar := languageEnvironment leadingChar.
	self withIndexDo: [:each :idx |
		each asciiValue > 255
			ifTrue: [self at: idx put: (Character leadingChar: leadingChar code: each asUnicode)]]! !

!String methodsFor: '*xml-parser' stamp: 'JAAyer 3/3/2010 02:03'!
splitQualifiedNameInto: aTwoArgumentBlock
	"Split the name into namespace prefix and local name (the block arguments)."

	| i |
	i := self lastIndexOf: $:.
	aTwoArgumentBlock
		value: ((i > 1) ifTrue: [self copyFrom: 1 to: (i - 1)] ifFalse: [nil])
		value: (self copyFrom: i + 1 to: self size)! !
XMLTokenizer initialize!
DTDEntityDeclaration initialize!
DTDParameterEntityDeclaration initialize!
DTDExternalEntityDeclaration initialize!
XMLWriter initialize!
